class ProfessionalColorTable(object):
 """
 Provides colors used for Microsoft Office display elements.

 

 ProfessionalColorTable()
 """
 ButtonCheckedGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used when the button is checked.



Get: ButtonCheckedGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ButtonCheckedGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used when the button is checked.



Get: ButtonCheckedGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ButtonCheckedGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used when the button is checked.



Get: ButtonCheckedGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 ButtonCheckedHighlight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color used when the button is checked.



Get: ButtonCheckedHighlight(self: ProfessionalColorTable) -> Color



"""

 ButtonCheckedHighlightBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use with System.Windows.Forms.ProfessionalColorTable.ButtonCheckedHighlight.



Get: ButtonCheckedHighlightBorder(self: ProfessionalColorTable) -> Color



"""

 ButtonPressedBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use with the System.Windows.Forms.ProfessionalColorTable.ButtonPressedGradientBegin,System.Windows.Forms.ProfessionalColorTable.ButtonPressedGradientMiddle,and System.Windows.Forms.ProfessionalColorTable.ButtonPressedGradientEnd colors.



Get: ButtonPressedBorder(self: ProfessionalColorTable) -> Color



"""

 ButtonPressedGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used when the button is pressed.



Get: ButtonPressedGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ButtonPressedGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used when the button is pressed.



Get: ButtonPressedGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ButtonPressedGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used when the button is pressed.



Get: ButtonPressedGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 ButtonPressedHighlight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color used when the button is pressed.



Get: ButtonPressedHighlight(self: ProfessionalColorTable) -> Color



"""

 ButtonPressedHighlightBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use with System.Windows.Forms.ProfessionalColorTable.ButtonPressedHighlight.



Get: ButtonPressedHighlightBorder(self: ProfessionalColorTable) -> Color



"""

 ButtonSelectedBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use with the System.Windows.Forms.ProfessionalColorTable.ButtonSelectedGradientBegin,System.Windows.Forms.ProfessionalColorTable.ButtonSelectedGradientMiddle,and System.Windows.Forms.ProfessionalColorTable.ButtonSelectedGradientEnd colors.



Get: ButtonSelectedBorder(self: ProfessionalColorTable) -> Color



"""

 ButtonSelectedGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used when the button is selected.



Get: ButtonSelectedGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ButtonSelectedGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used when the button is selected.



Get: ButtonSelectedGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ButtonSelectedGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used when the button is selected.



Get: ButtonSelectedGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 ButtonSelectedHighlight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color used when the button is selected.



Get: ButtonSelectedHighlight(self: ProfessionalColorTable) -> Color



"""

 ButtonSelectedHighlightBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use with System.Windows.Forms.ProfessionalColorTable.ButtonSelectedHighlight.



Get: ButtonSelectedHighlightBorder(self: ProfessionalColorTable) -> Color



"""

 CheckBackground=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color to use when the button is checked and gradients are being used.



Get: CheckBackground(self: ProfessionalColorTable) -> Color



"""

 CheckPressedBackground=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color to use when the button is checked and selected and gradients are being used.



Get: CheckPressedBackground(self: ProfessionalColorTable) -> Color



"""

 CheckSelectedBackground=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color to use when the button is checked and selected and gradients are being used.



Get: CheckSelectedBackground(self: ProfessionalColorTable) -> Color



"""

 GripDark=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the color to use for shadow effects on the grip (move handle).



Get: GripDark(self: ProfessionalColorTable) -> Color



"""

 GripLight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the color to use for highlight effects on the grip (move handle).



Get: GripLight(self: ProfessionalColorTable) -> Color



"""

 ImageMarginGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the image margin of a System.Windows.Forms.ToolStripDropDownMenu.



Get: ImageMarginGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ImageMarginGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the image margin of a System.Windows.Forms.ToolStripDropDownMenu.



Get: ImageMarginGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ImageMarginGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used in the image margin of a System.Windows.Forms.ToolStripDropDownMenu.



Get: ImageMarginGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 ImageMarginRevealedGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the image margin of a System.Windows.Forms.ToolStripDropDownMenu when an item is revealed.



Get: ImageMarginRevealedGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ImageMarginRevealedGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the image margin of a System.Windows.Forms.ToolStripDropDownMenu when an item is revealed.



Get: ImageMarginRevealedGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ImageMarginRevealedGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used in the image margin of a System.Windows.Forms.ToolStripDropDownMenu when an item is revealed.



Get: ImageMarginRevealedGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 MenuBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the color that is the border color to use on a System.Windows.Forms.MenuStrip.



Get: MenuBorder(self: ProfessionalColorTable) -> Color



"""

 MenuItemBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use with a System.Windows.Forms.ToolStripMenuItem.



Get: MenuItemBorder(self: ProfessionalColorTable) -> Color



"""

 MenuItemPressedGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used when a top-level System.Windows.Forms.ToolStripMenuItem is pressed.



Get: MenuItemPressedGradientBegin(self: ProfessionalColorTable) -> Color



"""

 MenuItemPressedGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used when a top-level System.Windows.Forms.ToolStripMenuItem is pressed.



Get: MenuItemPressedGradientEnd(self: ProfessionalColorTable) -> Color



"""

 MenuItemPressedGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used when a top-level System.Windows.Forms.ToolStripMenuItem is pressed.



Get: MenuItemPressedGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 MenuItemSelected=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid color to use when a System.Windows.Forms.ToolStripMenuItem other than the top-level System.Windows.Forms.ToolStripMenuItem is selected.



Get: MenuItemSelected(self: ProfessionalColorTable) -> Color



"""

 MenuItemSelectedGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used when the System.Windows.Forms.ToolStripMenuItem is selected.



Get: MenuItemSelectedGradientBegin(self: ProfessionalColorTable) -> Color



"""

 MenuItemSelectedGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used when the System.Windows.Forms.ToolStripMenuItem is selected.



Get: MenuItemSelectedGradientEnd(self: ProfessionalColorTable) -> Color



"""

 MenuStripGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the System.Windows.Forms.MenuStrip.



Get: MenuStripGradientBegin(self: ProfessionalColorTable) -> Color



"""

 MenuStripGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the System.Windows.Forms.MenuStrip.



Get: MenuStripGradientEnd(self: ProfessionalColorTable) -> Color



"""

 OverflowButtonGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the System.Windows.Forms.ToolStripOverflowButton.



Get: OverflowButtonGradientBegin(self: ProfessionalColorTable) -> Color



"""

 OverflowButtonGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the System.Windows.Forms.ToolStripOverflowButton.



Get: OverflowButtonGradientEnd(self: ProfessionalColorTable) -> Color



"""

 OverflowButtonGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used in the System.Windows.Forms.ToolStripOverflowButton.



Get: OverflowButtonGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 RaftingContainerGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the System.Windows.Forms.ToolStripContainer.



Get: RaftingContainerGradientBegin(self: ProfessionalColorTable) -> Color



"""

 RaftingContainerGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the System.Windows.Forms.ToolStripContainer.



Get: RaftingContainerGradientEnd(self: ProfessionalColorTable) -> Color



"""

 SeparatorDark=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the color to use to for shadow effects on the System.Windows.Forms.ToolStripSeparator.



Get: SeparatorDark(self: ProfessionalColorTable) -> Color



"""

 SeparatorLight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the color to use to for highlight effects on the System.Windows.Forms.ToolStripSeparator.



Get: SeparatorLight(self: ProfessionalColorTable) -> Color



"""

 StatusStripGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used on the System.Windows.Forms.StatusStrip.



Get: StatusStripGradientBegin(self: ProfessionalColorTable) -> Color



"""

 StatusStripGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used on the System.Windows.Forms.StatusStrip.



Get: StatusStripGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ToolStripBorder=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the border color to use on the bottom edge of the System.Windows.Forms.ToolStrip.



Get: ToolStripBorder(self: ProfessionalColorTable) -> Color



"""

 ToolStripContentPanelGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the System.Windows.Forms.ToolStripContentPanel.



Get: ToolStripContentPanelGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ToolStripContentPanelGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the System.Windows.Forms.ToolStripContentPanel.



Get: ToolStripContentPanelGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ToolStripDropDownBackground=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the solid background color of the System.Windows.Forms.ToolStripDropDown.



Get: ToolStripDropDownBackground(self: ProfessionalColorTable) -> Color



"""

 ToolStripGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the System.Windows.Forms.ToolStrip background.



Get: ToolStripGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ToolStripGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the System.Windows.Forms.ToolStrip background.



Get: ToolStripGradientEnd(self: ProfessionalColorTable) -> Color



"""

 ToolStripGradientMiddle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the middle color of the gradient used in the System.Windows.Forms.ToolStrip background.



Get: ToolStripGradientMiddle(self: ProfessionalColorTable) -> Color



"""

 ToolStripPanelGradientBegin=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting color of the gradient used in the System.Windows.Forms.ToolStripPanel.



Get: ToolStripPanelGradientBegin(self: ProfessionalColorTable) -> Color



"""

 ToolStripPanelGradientEnd=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the end color of the gradient used in the System.Windows.Forms.ToolStripPanel.



Get: ToolStripPanelGradientEnd(self: ProfessionalColorTable) -> Color



"""

 UseSystemColors=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether to use System.Drawing.SystemColors rather than colors that match the current visual style.



Get: UseSystemColors(self: ProfessionalColorTable) -> bool



Set: UseSystemColors(self: ProfessionalColorTable)=value

"""





class ToolStripManager(object):
 """ Controls System.Windows.Forms.ToolStrip rendering and rafting,and the merging of System.Windows.Forms.MenuStrip,System.Windows.Forms.ToolStripDropDownMenu,and System.Windows.Forms.ToolStripMenuItem objects. This class cannot be inherited. """
 @staticmethod
 def FindToolStrip(toolStripName):
  """
  FindToolStrip(toolStripName: str) -> ToolStrip

  

   Finds the specified System.Windows.Forms.ToolStrip or a type derived from 

    System.Windows.Forms.ToolStrip.

  

  

   toolStripName: A string specifying the name of the System.Windows.Forms.ToolStrip or derived 

    System.Windows.Forms.ToolStrip type to find.

  

   Returns: The System.Windows.Forms.ToolStrip or one of its derived types as specified by the toolStripName 

    parameter,or null if the System.Windows.Forms.ToolStrip is not found.
  """
  pass
 @staticmethod
 def IsShortcutDefined(shortcut):
  """
  IsShortcutDefined(shortcut: Keys) -> bool

  

   Retrieves a value indicating whether the specified shortcut key is used by any of the 

    System.Windows.Forms.ToolStrip controls of a form.

  

  

   shortcut: The shortcut key for which to search.

   Returns: true if the shortcut key is used by any System.Windows.Forms.ToolStrip on the form; otherwise,

    false.
  """
  pass
 @staticmethod
 def IsValidShortcut(shortcut):
  """
  IsValidShortcut(shortcut: Keys) -> bool

  

   Retrieves a value indicating whether a defined shortcut key is valid.

  

   shortcut: The shortcut key to test for validity.

   Returns: true if the shortcut key is valid; otherwise,false.
  """
  pass
 @staticmethod
 def LoadSettings(targetForm,key=None):
  """
  LoadSettings(targetForm: Form,key: str)

   Loads settings for the specified System.Windows.Forms.Form using the specified settings key.

  

   targetForm: The System.Windows.Forms.Form for which to load settings.

   key: A System.String representing the settings key for this System.Windows.Forms.Form.

  LoadSettings(targetForm: Form)

   Loads settings for the given System.Windows.Forms.Form using the full name of the 

    System.Windows.Forms.Form as the settings key.

  

  

   targetForm: The System.Windows.Forms.Form whose name is also the settings key.
  """
  pass
 @staticmethod
 def Merge(sourceToolStrip,*__args):
  """
  Merge(sourceToolStrip: ToolStrip,targetName: str) -> bool

  

   Combines two System.Windows.Forms.ToolStrip objects of the same type.

  

   sourceToolStrip: The System.Windows.Forms.ToolStrip to be combined with the System.Windows.Forms.ToolStrip 

    referred to by the targetName parameter.

  

   targetName: The name of the System.Windows.Forms.ToolStrip that receives the System.Windows.Forms.ToolStrip 

    referred to by the sourceToolStrip parameter.

  

   Returns: true if the merge is successful; otherwise,false.

  Merge(sourceToolStrip: ToolStrip,targetToolStrip: ToolStrip) -> bool

  

   Combines two System.Windows.Forms.ToolStrip objects of different types.

  

   sourceToolStrip: The System.Windows.Forms.ToolStrip to be combined with the System.Windows.Forms.ToolStrip 

    referred to by the targetToolStrip parameter.

  

   targetToolStrip: The System.Windows.Forms.ToolStrip that receives the System.Windows.Forms.ToolStrip referred to 

    by the sourceToolStrip parameter.

  

   Returns: true if the merge is successful; otherwise,false.
  """
  pass
 @staticmethod
 def RevertMerge(*__args):
  """
  RevertMerge(targetName: str) -> bool

  

   Undoes a merging of two System.Windows.Forms.ToolStrip objects,returning the 

    System.Windows.Forms.ToolStrip with the specified name to its state before the merge and 

    nullifying all previous merge operations.

  

  

   targetName: The name of the System.Windows.Forms.ToolStripItem for which to undo a merge operation.

   Returns: true if the undoing of the merge is successful; otherwise,false.

  RevertMerge(targetToolStrip: ToolStrip,sourceToolStrip: ToolStrip) -> bool

  

   Undoes a merging of two System.Windows.Forms.ToolStrip objects,returning both 

    System.Windows.Forms.ToolStrip controls to their state before the merge and nullifying all 

    previous merge operations.

  

  

   targetToolStrip: The name of the System.Windows.Forms.ToolStripItem for which to undo a merge operation.

   sourceToolStrip: The System.Windows.Forms.ToolStrip that was merged with the targetToolStrip.

   Returns: true if the undoing of the merge is successful; otherwise,false.

  RevertMerge(targetToolStrip: ToolStrip) -> bool

  

   Undoes a merging of two System.Windows.Forms.ToolStrip objects,returning the specified 

    System.Windows.Forms.ToolStrip to its state before the merge and nullifying all previous merge 

    operations.

  

  

   targetToolStrip: The System.Windows.Forms.ToolStripItem for which to undo a merge operation.

   Returns: true if the undoing of the merge is successful; otherwise,false.
  """
  pass
 @staticmethod
 def SaveSettings(sourceForm,key=None):
  """
  SaveSettings(sourceForm: Form,key: str)

   Saves settings for the specified System.Windows.Forms.Form using the specified settings key.

  

   sourceForm: The System.Windows.Forms.Form for which to save settings.

   key: A System.String representing the settings key for this System.Windows.Forms.Form.

  SaveSettings(sourceForm: Form)

   Saves settings for the given System.Windows.Forms.Form using the full name of the 

    System.Windows.Forms.Form as the settings key.

  

  

   sourceForm: The System.Windows.Forms.Form whose name is also the settings key.
  """
  pass
 Renderer=None
 RendererChanged=None
 RenderMode=None
 VisualStylesEnabled=False




class IDataGridViewEditingCell:
 """ Defines common functionality for a cell that allows the manipulation of its value. """
 def GetEditingCellFormattedValue(self,context):
  """
  GetEditingCellFormattedValue(self: IDataGridViewEditingCell,context: DataGridViewDataErrorContexts) -> object

  

   Retrieves the formatted value of the cell.

  

   context: A bitwise combination of System.Windows.Forms.DataGridViewDataErrorContexts values that 

    specifies the context in which the data is needed.

  

   Returns: An System.Object that represents the formatted version of the cell contents.
  """
  pass
 def PrepareEditingCellForEdit(self,selectAll):
  """
  PrepareEditingCellForEdit(self: IDataGridViewEditingCell,selectAll: bool)

   Prepares the currently selected cell for editing

  

   selectAll: true to select the cell contents; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 EditingCellFormattedValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the formatted value of the cell.



Get: EditingCellFormattedValue(self: IDataGridViewEditingCell) -> object



Set: EditingCellFormattedValue(self: IDataGridViewEditingCell)=value

"""

 EditingCellValueChanged=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the value of the cell has changed.



Get: EditingCellValueChanged(self: IDataGridViewEditingCell) -> bool



Set: EditingCellValueChanged(self: IDataGridViewEditingCell)=value

"""





class FieldOffsetAttribute(Attribute,_Attribute):
 """
 Indicates the physical position of fields within the unmanaged representation of a class or structure.

 

 FieldOffsetAttribute(offset: int)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,offset):
  """ __new__(cls: type,offset: int) """
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the offset from the beginning of the structure to the beginning of the field.



Get: Value(self: FieldOffsetAttribute) -> int



"""





class EventTypeFilter(TraceFilter):
 """
 Indicates whether a listener should trace based on the event type.

 

 EventTypeFilter(level: SourceLevels)
 """
 def ShouldTrace(self,cache,source,eventType,id,formatOrMessage,args,data1,data):
  """
  ShouldTrace(self: EventTypeFilter,cache: TraceEventCache,source: str,eventType: TraceEventType,id: int,formatOrMessage: str,args: Array[object],data1: object,data: Array[object]) -> bool

  

   Determines whether the trace listener should trace the event.

  

   cache: A System.Diagnostics.TraceEventCache that represents the information cache for the trace event.

   source: The name of the source.

   eventType: One of the System.Diagnostics.TraceEventType values.

   id: A trace identifier number.

   formatOrMessage: The format to use for writing an array of arguments,or a message to write.

   args: An array of argument objects.

   data1: A trace data object.

   data: An array of trace data objects.

   Returns: trueif the trace should be produced; otherwise,false.
  """
  pass
 @staticmethod
 def __new__(self,level):
  """ __new__(cls: type,level: SourceLevels) """
  pass
 EventType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the event type of the messages to trace.



Get: EventType(self: EventTypeFilter) -> SourceLevels



Set: EventType(self: EventTypeFilter)=value

"""





class CacheVirtualItemsEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.ListView.CacheVirtualItems event.

 

 CacheVirtualItemsEventArgs(startIndex: int,endIndex: int)
 """
 @staticmethod
 def __new__(self,startIndex,endIndex):
  """ __new__(cls: type,startIndex: int,endIndex: int) """
  pass
 EndIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the ending index for the range of values needed by a System.Windows.Forms.ListView control in virtual mode.



Get: EndIndex(self: CacheVirtualItemsEventArgs) -> int



"""

 StartIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the starting index for a range of values needed by a System.Windows.Forms.ListView control in virtual mode.



Get: StartIndex(self: CacheVirtualItemsEventArgs) -> int



"""





"""This module manages the telemetry system.

This function is used to setup the telemetry system on pyRevit startup:
    ```python
    setup_telemetry()
    ```

These functions are used to query information about the logging system:
    ```python
    get_telemetry_state()
    ```

    ```python
    get_apptelemetry_state()
    ```

This module also provides a wrapper class around the command results
dictionary that is included with the telemetry record.

Scripts should use the instance of this class provided by the
script module. See `script.get_results()` for examples
"""
import os.path as op
import json

from pyrevit import HOST_APP, PYREVIT_VERSION_APP_DIR, PYREVIT_FILE_PREFIX
from pyrevit.runtime.types import EventType, EventTelemetry
from pyrevit import coreutils
from pyrevit.coreutils.logger import get_logger
from pyrevit.coreutils import envvars
from pyrevit.compat import urlopen

from pyrevit.loader import sessioninfo
from pyrevit.userconfig import user_config

from pyrevit.labs import PyRevit

from pyrevit.telemetry import events as telemetry_events


# templates for telemetry file naming
FILE_LOG_EXT = 'json'
FILE_LOG_FILENAME_TEMPLATE = '{}_{}_telemetry.{}'


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)

# pyrevit global consts
consts = PyRevit.PyRevitConsts


def get_default_telemetry_filepath():
    return PYREVIT_VERSION_APP_DIR


def get_telemetry_state():
    return envvars.get_pyrevit_env_var(envvars.TELEMETRYSTATE_ENVVAR)


def get_telemetry_utc_timestamp():
    return envvars.get_pyrevit_env_var(envvars.TELEMETRYUTCTIMESTAMPS_ENVVAR)


def get_telemetry_file_dir():
    return envvars.get_pyrevit_env_var(envvars.TELEMETRYDIR_ENVVAR)


def get_telemetry_file_path():
    return envvars.get_pyrevit_env_var(envvars.TELEMETRYFILE_ENVVAR)


def get_telemetry_server_url():
    return envvars.get_pyrevit_env_var(envvars.TELEMETRYSERVER_ENVVAR)


def get_telemetry_include_hooks():
    return envvars.get_pyrevit_env_var(envvars.TELEMETRYINCLUDEHOOKS_ENVVAR)


def set_telemetry_state(state):
    envvars.set_pyrevit_env_var(envvars.TELEMETRYSTATE_ENVVAR, state)
    user_config.telemetry_status = state


def set_telemetry_utc_timestamp(state):
    envvars.set_pyrevit_env_var(envvars.TELEMETRYUTCTIMESTAMPS_ENVVAR, state)
    user_config.telemetry_utc_timestamp = state


def set_telemetry_file_dir(file_dir):
    if not file_dir or not op.isdir(file_dir):
        disable_telemetry_to_file()
    envvars.set_pyrevit_env_var(envvars.TELEMETRYDIR_ENVVAR, file_dir)
    user_config.telemetry_file_dir = file_dir


def set_telemetry_file_path(file_path):
    envvars.set_pyrevit_env_var(envvars.TELEMETRYFILE_ENVVAR, file_path)


def set_telemetry_server_url(server_url):
    envvars.set_pyrevit_env_var(envvars.TELEMETRYSERVER_ENVVAR, server_url)
    user_config.telemetry_server_url = server_url


def set_telemetry_include_hooks(state):
    envvars.set_pyrevit_env_var(envvars.TELEMETRYINCLUDEHOOKS_ENVVAR, state)
    user_config.telemetry_include_hooks = state


def disable_telemetry():
    set_telemetry_state(False)


def disable_telemetry_to_file():
    set_telemetry_file_path('')


def disable_telemetry_to_server():
    set_telemetry_server_url('')


def get_apptelemetry_state():
    return envvars.get_pyrevit_env_var(envvars.APPTELEMETRYSTATE_ENVVAR)


def set_apptelemetry_state(state):
    envvars.set_pyrevit_env_var(envvars.APPTELEMETRYSTATE_ENVVAR, state)
    user_config.apptelemetry_status = state


def get_apptelemetry_handler():
    return envvars.get_pyrevit_env_var(envvars.APPTELEMETRYHANDLER_ENVVAR)


def set_apptelemetry_handler(handler):
    envvars.set_pyrevit_env_var(envvars.APPTELEMETRYHANDLER_ENVVAR, handler)


def get_apptelemetry_server_url():
    return envvars.get_pyrevit_env_var(envvars.APPTELEMETRYSERVER_ENVVAR)


def get_apptelemetry_event_flags():
    # default value is 16 bytes of 0
    flags_hex = \
        user_config.apptelemetry_event_flags or '0x00000000000000000000000000000000'
    return coreutils.hex2int_long(flags_hex)


def set_apptelemetry_server_url(server_url):
    envvars.set_pyrevit_env_var(envvars.APPTELEMETRYSERVER_ENVVAR, server_url)
    user_config.apptelemetry_server_url = server_url


def set_apptelemetry_event_flags(event_flags):
    flags_hex = coreutils.int2hex_long(event_flags)
    user_config.apptelemetry_event_flags = flags_hex
    envvars.set_pyrevit_env_var(
        envvars.APPTELEMETRYEVENTFLAGS_ENVVAR, flags_hex)


def disable_apptelemetry():
    set_apptelemetry_state(False)


def disable_apptelemetry_to_server():
    set_apptelemetry_server_url('')


def get_apptelemetry_event_types():
    return list(coreutils.get_enum_values(EventType))


def get_apptelemetry_event_state(flags, event_type):
    event_idx = get_apptelemetry_event_types().index(event_type)
    return flags & (1<<event_idx)


def set_apptelemetry_event_state(flags, event_type):
    return flags | (1<<int(event_type))


def unset_apptelemetry_event_state(flags, event_type):
    return flags & ~(1<<int(event_type))


def _setup_default_logfile(telemetry_fullpath):
    # setup default telemetry file name
    if not op.exists(telemetry_fullpath):
        # if file does not exist, let's write the basic JSON list to it.
        with open(telemetry_fullpath, 'w') as log_file:
            log_file.write('[]')


def get_status_from_url(server_url):
    server_url = server_url.lower()
    if server_url.endswith('scripts/'):
        server_url = server_url.replace('scripts/', 'status')
    elif server_url.endswith('events/'):
        server_url = server_url.replace('events/', 'status')

    try:
        return json.loads(urlopen(server_url).read())
    except Exception:
        return None


def get_status():
    return get_status_from_url(
        get_telemetry_server_url()
        or get_apptelemetry_server_url()
        )


def setup_telemetry(session_id=None):
    """Sets up the telemetry default config and environment values."""
    # make sure session id is availabe
    if not session_id:
        session_id = sessioninfo.get_session_uuid()

    # PYREVIT TELEMETRY -------------------------------------------------------
    # utc timestamp
    telemetry_utc_timestamp = user_config.telemetry_utc_timestamp
    set_telemetry_utc_timestamp(telemetry_utc_timestamp)

    # global telemetry toggle
    telemetry_state = user_config.telemetry_status
    set_telemetry_state(telemetry_state)

    # read or setup default values for file telemetry
    # default file path and name for telemetry
    telemetry_file_dir = user_config.telemetry_file_dir
    set_telemetry_file_dir(telemetry_file_dir)

    # check file telemetry config and setup destination
    if not telemetry_file_dir or coreutils.is_blank(telemetry_file_dir):
        # if no config is provided, disable output
        disable_telemetry_to_file()
    # if config exists, create new telemetry file under the same address
    elif telemetry_state:
        if op.isdir(telemetry_file_dir):
            telemetry_file_name = \
                FILE_LOG_FILENAME_TEMPLATE.format(PYREVIT_FILE_PREFIX,
                                                  session_id,
                                                  FILE_LOG_EXT)
            # if directory is valid
            telemetry_fullfilepath = \
                op.join(telemetry_file_dir, telemetry_file_name)
            set_telemetry_file_path(telemetry_fullfilepath)
            # setup telemetry file or disable if failed
            try:
                _setup_default_logfile(telemetry_fullfilepath)
            except Exception as write_err:
                mlogger.error('Telemetry is active but log file location '
                              'is not accessible. | %s', write_err)
                disable_telemetry_to_file()
        else:
            # if not, show error and disable telemetry
            mlogger.error('Provided telemetry address does not exits or is '
                          'not a directory. Telemetry disabled.')
            disable_telemetry_to_file()

    # read or setup default values for server telemetry
    telemetry_server_url = user_config.telemetry_server_url

    # check server telemetry config and setup destination
    if not telemetry_server_url or coreutils.is_blank(telemetry_server_url):
        # if no config is provided, disable output
        disable_telemetry_to_server()
    else:
        # if config exists, setup server logging
        set_telemetry_server_url(telemetry_server_url)

    # set telemetry script types
    set_telemetry_include_hooks(user_config.telemetry_include_hooks)

    # APP TELEMETRY ------------------------------------------------------------
    # setup default value for telemetry global switch
    apptelemetry_state = user_config.apptelemetry_status
    set_apptelemetry_state(apptelemetry_state)

    # read or setup default values for server telemetry
    apptelemetry_server_url = user_config.apptelemetry_server_url

    # check server telemetry config and setup destination
    if not apptelemetry_server_url \
            or coreutils.is_blank(apptelemetry_server_url):
        # if no config is provided, disable output
        disable_apptelemetry_to_server()
    else:
        # if config exists, setup server logging
        set_apptelemetry_server_url(apptelemetry_server_url)

    # setup events
    new_telemetry_handler = EventTelemetry(session_id, HOST_APP.username)
    telemetry_handler = get_apptelemetry_handler()
    if telemetry_handler:
        # clear existing
        telemetry_events.unregister_all_event_telemetries(telemetry_handler)

    set_apptelemetry_handler(new_telemetry_handler)
    # register handlers only if telemetry system is active
    if apptelemetry_state:
        apptelemetry_event_flags = get_apptelemetry_event_flags()
        # re-register events with new telemetry_handler
        telemetry_events.register_event_telemetry(
            new_telemetry_handler,
            apptelemetry_event_flags
        )

    user_config.save_changes()



class AutoCompleteSource(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the source for System.Windows.Forms.ComboBox and System.Windows.Forms.TextBox automatic completion functionality.

 

 enum AutoCompleteSource,values: AllSystemSources (7),AllUrl (6),CustomSource (64),FileSystem (1),FileSystemDirectories (32),HistoryList (2),ListItems (256),None (128),RecentlyUsedList (4)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 AllSystemSources=None
 AllUrl=None
 CustomSource=None
 FileSystem=None
 FileSystemDirectories=None
 HistoryList=None
 ListItems=None
 None=None
 RecentlyUsedList=None
 value__=None




class WebBrowserSiteBase(object,IOleControlSite,IOleClientSite,IOleInPlaceSite,ISimpleFrameSite,IPropertyNotifySink,IDisposable):
 """ Implements the interfaces of an ActiveX site for use as a base class by the System.Windows.Forms.WebBrowser.WebBrowserSite class. """
 def Dispose(self):
  """
  Dispose(self: WebBrowserSiteBase)

   Releases all resources used by the System.Windows.Forms.WebBrowserSiteBase.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass



"""pyRevit root level config for all pyrevit sub-modules.

Examples:
        ```python
        from pyrevit import DB, UI
        from pyrevit import PyRevitException, PyRevitIOError

        # pyrevit module has global instance of the
        # _HostAppPostableCommand and _ExecutorParams classes already created
        # import and use them like below
        from pyrevit import HOST_APP
        from pyrevit import EXEC_PARAMS
        ```
"""
#pylint: disable=W0703,C0302,C0103,C0413,raise-missing-from
import sys
import os
import os.path as op
from collections import namedtuple
import traceback
import re
from pyrevit.compat import PY3

import clr  # pylint: disable=E0401

import System

from pyrevit import compat

PYREVIT_ADDON_NAME = 'pyRevit'
PYREVIT_CLI_NAME = 'pyrevit.exe'

# extract version from version file
VERSION_STRING = '0.0.'
if PY3:
    with open(op.join(op.dirname(__file__), 'version'), 'r', encoding='utf-8') as version_file:
        VERSION_STRING = version_file.read()
else:
    with open(op.join(op.dirname(__file__), 'version'), 'r') as version_file:
        VERSION_STRING = version_file.read()
matches = re.findall(r'(\d+)\.(\d+)\.(\d+)\.?(.+)?', VERSION_STRING)[0]
if len(matches) == 4:
    VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH, BUILD_METADATA = matches
else:
    VERSION_MAJOR, VERSION_MINOR, VERSION_PATCH = matches
    BUILD_METADATA = ""

try:
    VERSION_MAJOR = int(VERSION_MAJOR)
    VERSION_MINOR = int(VERSION_MINOR)
    VERSION_PATCH = int(VERSION_PATCH)
except:
    raise Exception('Critical Error. Can not determine pyRevit version.')
# -----------------------------------------------------------------------------
# config environment paths
# -----------------------------------------------------------------------------
# main pyrevit repo folder
try:
    # 3 steps back for <home>/Lib/pyrevit
    HOME_DIR = op.dirname(op.dirname(op.dirname(__file__)))
except NameError:
    raise Exception('Critical Error. Can not find home directory.')

# try get net folder
DOTNET_RUNTIME_ID = "netcore" if compat.NETCORE else "netfx"

# BIN directory
ROOT_BIN_DIR = op.join(HOME_DIR, 'bin')
BIN_DIR = op.join(ROOT_BIN_DIR, DOTNET_RUNTIME_ID)

# main pyrevit lib folders
MAIN_LIB_DIR = op.join(HOME_DIR, 'pyrevitlib')
MISC_LIB_DIR = op.join(HOME_DIR, 'site-packages')

# path to pyrevit module
MODULE_DIR = op.join(MAIN_LIB_DIR, 'pyrevit')

# loader directory
LOADER_DIR = op.join(MODULE_DIR, 'loader')

# runtime directory
RUNTIME_DIR = op.join(MODULE_DIR, 'runtime')

# addin directory
ADDIN_DIR = op.join(LOADER_DIR, 'addin')

# if loader module is available means pyRevit is being executed by Revit.
import pyrevit.engine as eng
if eng.EngineVersion != 000:
    ENGINES_DIR = op.join(BIN_DIR, 'engines', eng.EngineVersion)
# otherwise it might be under test, or documentation processing.
# so let's keep the symbols but set to None (fake the symbols)
else:
    ENGINES_DIR = None

# add the framework dll path to the search paths
sys.path.append(BIN_DIR)
sys.path.append(ADDIN_DIR)
sys.path.append(ENGINES_DIR)


PYREVIT_CLI_PATH = op.join(HOME_DIR, 'bin', PYREVIT_CLI_NAME)


# now we can start importing stuff
from pyrevit.compat import safe_strtype
from pyrevit.framework import Process
from pyrevit.framework import Windows
from pyrevit.framework import Forms
from pyrevit import api
from pyrevit.api import DB, UI, ApplicationServices, AdWindows

# -----------------------------------------------------------------------------
# Base Exceptions
# -----------------------------------------------------------------------------
TRACEBACK_TITLE = 'Traceback:'


# General Exceptions
class PyRevitException(Exception):
    """Common base class for all pyRevit exceptions.

    Parameters args and message are derived from Exception class.
    """

    @property
    def msg(self):
        """Return exception message."""
        if self.args:
            return self.args[0] #pylint: disable=E1136
        else:
            return ''

    def __repr__(self):
        return str(self)

    def __str__(self):
        """Process stack trace and prepare report for output window."""
        sys.exc_type, sys.exc_value, sys.exc_traceback = sys.exc_info()
        try:
            tb_report = traceback.format_tb(sys.exc_traceback)[0]
            if self.msg:
                return '{}\n\n{}\n{}'.format(self.msg,
                                             TRACEBACK_TITLE,
                                             tb_report)
            else:
                return '{}\n{}'.format(TRACEBACK_TITLE, tb_report)
        except Exception:
            return Exception.__str__(self)


class PyRevitIOError(PyRevitException):
    """Common base class for all pyRevit io-related exceptions."""


class PyRevitCPythonNotSupported(PyRevitException):
    """Exception for features not supported under CPython."""
    def __init__(self, feature_name):
        super(PyRevitCPythonNotSupported, self).__init__()
        self.feature_name = feature_name

    def __str__(self):
        return self.msg

    @property
    def msg(self):
        """Return exception message."""
        return '\"{}\" is not currently supported under CPython' \
                .format(self.feature_name)


# -----------------------------------------------------------------------------
# Wrapper for __revit__ builtin parameter set in scope by C# Script Executor
# -----------------------------------------------------------------------------
# namedtuple for passing information about a PostableCommand
_HostAppPostableCommand = namedtuple('_HostAppPostableCommand',
                                     ['name', 'key', 'id', 'rvtobj'])
"""Private namedtuple for passing information about a PostableCommand

Attributes:
    name (str): Postable command name
    key (str): Postable command key string
    id (int): Postable command id
    rvtobj (``RevitCommandId``): Postable command Id Object
"""


class _HostApplication(object):
    """Private Wrapper for Current Instance of Revit.

    Provides version info and comparison functionality, alongside providing
    info on the active screen, active document and ui-document, available
    postable commands, and other functionality.

    Examples:
            ```python
            hostapp = _HostApplication()
            hostapp.is_newer_than(2017)
            ```
    """

    def __init__(self):
        self._postable_cmds = []

    @property
    def uiapp(self):
        """Return UIApplication provided to the running command."""
        if isinstance(__revit__, UI.UIApplication):  #pylint: disable=undefined-variable
            return __revit__  #pylint: disable=undefined-variable

    @property
    def app(self):
        """Return Application provided to the running command."""
        if self.uiapp:
            return self.uiapp.Application
        elif isinstance(__revit__, ApplicationServices.Application):  #pylint: disable=undefined-variable
            return __revit__  #pylint: disable=undefined-variable

    @property
    def addin_id(self):
        """Return active addin id."""
        return self.app.ActiveAddInId

    @property
    def has_api_context(self):
        """Determine if host application is in API context."""
        return self.app.ActiveAddInId is not None

    @property
    def uidoc(self):
        """Return active UIDocument."""
        return getattr(self.uiapp, 'ActiveUIDocument', None)

    @property
    def doc(self):
        """Return active Document."""
        return getattr(self.uidoc, 'Document', None)

    @property
    def active_view(self):
        """Return view that is active (UIDocument.ActiveView)."""
        return getattr(self.uidoc, 'ActiveView', None)

    @active_view.setter
    def active_view(self, value):
        """Set the active view in user interface."""
        setattr(self.uidoc, 'ActiveView', value)

    @property
    def docs(self):
        """Return :obj:`list` of open :obj:`Document` objects."""
        return getattr(self.app, 'Documents', None)

    @property
    def available_servers(self):
        """Return :obj:`list` of available Revit server names."""
        return list(self.app.GetRevitServerNetworkHosts())

    @property
    def version(self):
        """str: Return version number (e.g. '2018')."""
        return self.app.VersionNumber

    @property
    def subversion(self):
        """str: Return subversion number (e.g. '2018.3')."""
        if hasattr(self.app, 'SubVersionNumber'):
            return self.app.SubVersionNumber
        else:
            return '{}.0'.format(self.version)

    @property
    def version_name(self):
        """str: Return version name (e.g. 'Autodesk Revit 2018')."""
        return self.app.VersionName

    @property
    def build(self):
        """str: Return build number (e.g. '20170927_1515(x64)')."""
        if int(self.version) >= 2021:
            # uses labs module that is imported later in this code
            return labs.extract_build_from_exe(self.proc_path)
        else:
            return self.app.VersionBuild

    @property
    def serial_no(self):
        """str: Return serial number number (e.g. '569-09704828')."""
        return api.get_product_serial_number()

    @property
    def pretty_name(self):
        """Returns the pretty name of the host.

        Examples:
            Autodesk Revit 2019.2 build: 20190808_0900(x64)

        Returns:
            (str): Pretty name of the host
        """
        host_name = self.version_name
        if self.is_newer_than(2017):
            host_name = host_name.replace(self.version, self.subversion)
        return "%s build: %s" % (host_name, self.build)

    @property
    def is_demo(self):
        """bool: Determine if product is using demo license."""
        return api.is_product_demo()

    @property
    def language(self):
        """str: Return language type (e.g. 'LanguageType.English_USA')."""
        return self.app.Language

    @property
    def username(self):
        """str: Return the username from Revit API (Application.Username)."""
        uname = self.app.Username
        uname = uname.split('@')[0]  # if username is email
        # removing dots since username will be used in file naming
        uname = uname.replace('.', '')
        return uname

    @property
    def proc(self):
        """System.Diagnostics.Process: Return current process object."""
        return Process.GetCurrentProcess()

    @property
    def proc_id(self):
        """int: Return current process id."""
        return Process.GetCurrentProcess().Id

    @property
    def proc_name(self):
        """str: Return current process name."""
        return Process.GetCurrentProcess().ProcessName

    @property
    def proc_path(self):
        """str: Return file path for the current process main module."""
        return Process.GetCurrentProcess().MainModule.FileName

    @property
    def proc_window(self):
        """``intptr``: Return handle to current process window."""
        if self.is_newer_than(2019, or_equal=True):
            return self.uiapp.MainWindowHandle
        else:
            return AdWindows.ComponentManager.ApplicationWindow

    @property
    def proc_screen(self):
        """``intptr``: Return handle to screen hosting current process."""
        return Forms.Screen.FromHandle(self.proc_window)

    @property
    def proc_screen_workarea(self):
        """``System.Drawing.Rectangle``: Return screen working area."""
        screen = HOST_APP.proc_screen
        if screen:
            return screen.WorkingArea

    @property
    def proc_screen_scalefactor(self):
        """float: Return scaling for screen hosting current process."""
        screen = HOST_APP.proc_screen
        if screen:
            actual_wdith = Windows.SystemParameters.PrimaryScreenWidth
            scaled_width = screen.PrimaryScreen.WorkingArea.Width
            return abs(scaled_width / actual_wdith)

    def is_newer_than(self, version, or_equal=False):
        """bool: Return True if host app is newer than provided version.

        Args:
            version (str or int): version to check against.
            or_equal (bool): Whether to include `version` in the comparison
        """
        if or_equal:
            return int(self.version) >= int(version)
        else:
            return int(self.version) > int(version)

    def is_older_than(self, version):
        """bool: Return True if host app is older than provided version.

        Args:
            version (str or int): version to check against.
        """
        return int(self.version) < int(version)

    def is_exactly(self, version):
        """bool: Return True if host app is equal to provided version.

        Args:
            version (str or int): version to check against.
        """
        return int(self.version) == int(version)

    def get_postable_commands(self):
        """Return list of postable commands.

        Returns:
            (list[_HostAppPostableCommand]): postable commands.
        """
        # if list of postable commands is _not_ already created
        # make the list and store in instance parameter
        if not self._postable_cmds:
            for pc in UI.PostableCommand.GetValues(UI.PostableCommand):
                try:
                    rcid = UI.RevitCommandId.LookupPostableCommandId(pc)
                    self._postable_cmds.append(
                        # wrap postable command info in custom namedtuple
                        _HostAppPostableCommand(name=safe_strtype(pc),
                                                key=rcid.Name,
                                                id=rcid.Id,
                                                rvtobj=rcid)
                        )
                except Exception:
                    # if any error occured when querying postable command
                    # or its info, pass silently
                    pass

        return self._postable_cmds

    def post_command(self, command_id):
        """Request Revit to run a command.

        Args:
            command_id (str): command identifier e.g. ID_REVIT_SAVE_AS_TEMPLATE
        """
        command_id = UI.RevitCommandId.LookupCommandId(command_id)
        self.uiapp.PostCommand(command_id)



try:
    # Create an intance of host application wrapper
    # making sure __revit__ is available
    HOST_APP = _HostApplication()
except Exception:
    raise Exception('Critical Error: Host software is not supported. '
                    '(__revit__ handle is not available)')


# -----------------------------------------------------------------------------
# Wrapper to access builtin parameters set in scope by C# Script Executor
# -----------------------------------------------------------------------------
class _ExecutorParams(object):
    """Private Wrapper that provides runtime environment info."""

    @property   # read-only
    def exec_id(self):
        """Return execution unique id."""
        try:
            return __execid__
        except NameError:
            pass

    @property   # read-only
    def exec_timestamp(self):
        """Return execution timestamp."""
        try:
            return __timestamp__
        except NameError:
            pass

    @property   # read-only
    def engine_id(self):
        """Return engine id."""
        try:
            return __cachedengineid__
        except NameError:
            pass

    @property   # read-only
    def engine_ver(self):
        """str: Return PyRevitLoader.ScriptExecutor hardcoded version."""
        if eng.ScriptExecutor:
            return eng.ScriptExecutor.EngineVersion

    @property  # read-only
    def cached_engine(self):
        """bool: Check whether pyrevit is running on a cached engine."""
        try:
            return __cachedengine__
        except NameError:
            return False

    @property  # read-only
    def first_load(self):
        """bool: Check whether pyrevit is not running in pyrevit command."""
        # if no output window is set by the executor, it means that pyRevit
        # is loading at Revit startup (not reloading)
        return True if self.window_handle is None else False

    @property   # read-only
    def script_runtime(self):
        """``PyRevitLabs.PyRevit.Runtime.ScriptRuntime``: Return command."""
        try:
            return __scriptruntime__
        except NameError:
            return None

    @property   # read-only
    def output_stream(self):
        """Return ScriptIO."""
        if self.script_runtime:
            return self.script_runtime.OutputStream

    @property   # read-only
    def script_data(self):
        """Return ScriptRuntime.ScriptData."""
        if self.script_runtime:
            return self.script_runtime.ScriptData

    @property   # read-only
    def script_runtime_cfgs(self):
        """Return ScriptRuntime.ScriptRuntimeConfigs."""
        if self.script_runtime:
            return self.script_runtime.ScriptRuntimeConfigs

    @property   # read-only
    def engine_cfgs(self):
        """Return ScriptRuntime.ScriptRuntimeConfigs."""
        if self.script_runtime:
            return self.script_runtime.EngineConfigs

    @property
    def command_mode(self):
        """bool: Check if pyrevit is running in pyrevit command context."""
        return self.script_runtime is not None

    @property
    def event_sender(self):
        """``Object``: Return event sender object."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.EventSender

    @property
    def event_args(self):
        """``DB.RevitAPIEventArgs``: Return event arguments object."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.EventArgs

    @property
    def event_doc(self):
        """``DB.Document``: Return document set in event args if available."""
        if self.event_args:
            if hasattr(self.event_args, 'Document'):
                return getattr(self.event_args, 'Document')
            elif hasattr(self.event_args, 'ActiveDocument'):
                return getattr(self.event_args, 'ActiveDocument')
            elif hasattr(self.event_args, 'CurrentDocument'):
                return getattr(self.event_args, 'CurrentDocument')
            elif hasattr(self.event_args, 'GetDocument'):
                return self.event_args.GetDocument()

    @property   # read-only
    def needs_refreshed_engine(self):
        """bool: Check if command needs a newly refreshed IronPython engine."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.RefreshEngine
        else:
            return False

    @property   # read-only
    def debug_mode(self):
        """bool: Check if command is in debug mode."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.DebugMode
        else:
            return False

    @property   # read-only
    def config_mode(self):
        """bool: Check if command is in config mode."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.ConfigMode
        else:
            return False

    @property   # read-only
    def executed_from_ui(self):
        """bool: Check if command was executed from ui."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.ExecutedFromUI
        else:
            return False

    @property   # read-only
    def needs_clean_engine(self):
        """bool: Check if command needs a clean IronPython engine."""
        if self.engine_cfgs:
            return self.engine_cfgs.CleanEngine
        else:
            return False

    @property   # read-only
    def needs_fullframe_engine(self):
        """bool: Check if command needs a full-frame IronPython engine."""
        if self.engine_cfgs:
            return self.engine_cfgs.FullFrameEngine
        else:
            return False

    @property   # read-only
    def needs_persistent_engine(self):
        """bool: Check if command needs a persistent IronPython engine."""
        if self.engine_cfgs:
            return self.engine_cfgs.PersistentEngine
        else:
            return False

    @property   # read
    def window_handle(self):
        """Output window handle."""
        if self.script_runtime:
            return self.script_runtime.OutputWindow

    @property
    def command_data(self):
        """ExternalCommandData: Return current command data."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.CommandData

    @property
    def command_elements(self):
        """``DB.ElementSet``: Return elements passed to by Revit."""
        if self.script_runtime_cfgs:
            return self.script_runtime_cfgs.SelectedElements

    @property   # read-only
    def command_path(self):
        """str: Return current command path."""
        if '__commandpath__' in __builtins__ \
                and __builtins__['__commandpath__']:
            return __builtins__['__commandpath__']
        elif self.script_runtime:
            return op.dirname(self.script_runtime.ScriptData.ScriptPath)

    @property   # read-only
    def command_config_path(self):
        """str: Return current command config script path."""
        if '__configcommandpath__' in __builtins__ \
                and __builtins__['__configcommandpath__']:
            return __builtins__['__configcommandpath__']
        elif self.script_runtime:
            return op.dirname(self.script_runtime.ScriptData.ConfigScriptPath)

    @property   # read-only
    def command_name(self):
        """str: Return current command name."""
        if '__commandname__' in __builtins__ \
                and __builtins__['__commandname__']:
            return __builtins__['__commandname__']
        elif self.script_runtime:
            return self.script_runtime.ScriptData.CommandName

    @property   # read-only
    def command_bundle(self):
        """str: Return current command bundle name."""
        if '__commandbundle__' in __builtins__ \
                and __builtins__['__commandbundle__']:
            return __builtins__['__commandbundle__']
        elif self.script_runtime:
            return self.script_runtime.ScriptData.CommandBundle

    @property   # read-only
    def command_extension(self):
        """str: Return current command extension name."""
        if '__commandextension__' in __builtins__ \
                and __builtins__['__commandextension__']:
            return __builtins__['__commandextension__']
        elif self.script_runtime:
            return self.script_runtime.ScriptData.CommandExtension

    @property   # read-only
    def command_uniqueid(self):
        """str: Return current command unique id."""
        if '__commanduniqueid__' in __builtins__ \
                and __builtins__['__commanduniqueid__']:
            return __builtins__['__commanduniqueid__']
        elif self.script_runtime:
            return self.script_runtime.ScriptData.CommandUniqueId

    @property   # read-only
    def command_controlid(self):
        """str: Return current command control id."""
        if '__commandcontrolid__' in __builtins__ \
                and __builtins__['__commandcontrolid__']:
            return __builtins__['__commandcontrolid__']
        elif self.script_runtime:
            return self.script_runtime.ScriptData.CommandControlId

    @property   # read-only
    def command_uibutton(self):
        """str: Return current command ui button."""
        if '__uibutton__' in __builtins__ \
                and __builtins__['__uibutton__']:
            return __builtins__['__uibutton__']

    @property
    def result_dict(self):
        """``Dictionary<String, String>``: Return results dict for logging."""
        if self.script_runtime:
            return self.script_runtime.GetResultsDictionary()


# create an instance of _ExecutorParams wrapping current runtime.
EXEC_PARAMS = _ExecutorParams()


# -----------------------------------------------------------------------------
# type to safely get document instance from app or event args
# -----------------------------------------------------------------------------

class _DocsGetter(object):
    """Instance to safely get document from HOST_APP instance or EXEC_PARAMS."""

    @property
    def doc(self):
        """Active document."""
        return HOST_APP.doc or EXEC_PARAMS.event_doc

    @property
    def docs(self):
        """List of active documents."""
        return HOST_APP.docs


DOCS = _DocsGetter()

# -----------------------------------------------------------------------------
# config user environment paths
# -----------------------------------------------------------------------------
# user env paths
ALLUSER_PROGRAMDATA = os.getenv('programdata')
USER_ROAMING_DIR = os.getenv('appdata')
USER_SYS_TEMP = os.getenv('temp')
USER_DESKTOP = op.expandvars('%userprofile%\\desktop')

# verify directory per issue #369
if not USER_DESKTOP or not op.exists(USER_DESKTOP):
    USER_DESKTOP = USER_SYS_TEMP

# default extensions directory
EXTENSIONS_DEFAULT_DIR = op.join(HOME_DIR, 'extensions')
THIRDPARTY_EXTENSIONS_DEFAULT_DIR = \
    op.join(USER_ROAMING_DIR, PYREVIT_ADDON_NAME, 'Extensions')

# create paths for pyrevit files
# pyrevit file directory
PYREVIT_ALLUSER_APP_DIR = op.join(ALLUSER_PROGRAMDATA, PYREVIT_ADDON_NAME)
PYREVIT_APP_DIR = op.join(USER_ROAMING_DIR, PYREVIT_ADDON_NAME)
PYREVIT_VERSION_APP_DIR = op.join(PYREVIT_APP_DIR, HOST_APP.version)

# add runtime paths to sys.paths
# this will allow importing any dynamically compiled DLLs that
# would be placed under this paths.
for pyrvt_app_dir in [PYREVIT_APP_DIR,
                        PYREVIT_VERSION_APP_DIR,
                        THIRDPARTY_EXTENSIONS_DEFAULT_DIR]:
    if not op.isdir(pyrvt_app_dir):
        try:
            os.mkdir(pyrvt_app_dir)
            sys.path.append(pyrvt_app_dir)
        except Exception as err:
            raise PyRevitException('Can not access pyRevit '
                                    'folder at: {} | {}'
                                    .format(pyrvt_app_dir, err))
    else:
        sys.path.append(pyrvt_app_dir)


# -----------------------------------------------------------------------------
# standard prefixes for naming pyrevit files (config, appdata and temp files)
# -----------------------------------------------------------------------------
# e.g. pyRevit_
PYREVIT_FILE_PREFIX_UNIVERSAL = '{}_'.format(PYREVIT_ADDON_NAME)
PYREVIT_FILE_PREFIX_UNIVERSAL_REGEX = \
    r'^' + PYREVIT_ADDON_NAME + r'_(?P<fname>.+)'

# e.g. pyRevit_2018_
PYREVIT_FILE_PREFIX = '{}_{}_'.format(PYREVIT_ADDON_NAME,
                                        HOST_APP.version)
PYREVIT_FILE_PREFIX_REGEX = \
    r'^' + PYREVIT_ADDON_NAME + r'_(?P<version>\d{4})_(?P<fname>.+)'

# e.g. pyRevit_2018_14422_
PYREVIT_FILE_PREFIX_STAMPED = '{}_{}_{}_'.format(PYREVIT_ADDON_NAME,
                                                    HOST_APP.version,
                                                    HOST_APP.proc_id)
PYREVIT_FILE_PREFIX_STAMPED_REGEX = \
    r'^' + PYREVIT_ADDON_NAME \
    + r'_(?P<version>\d{4})_(?P<pid>\d+)_(?P<fname>.+)'

# e.g. pyRevit_eirannejad_
PYREVIT_FILE_PREFIX_UNIVERSAL_USER = '{}_{}_'.format(PYREVIT_ADDON_NAME,
                                                        HOST_APP.username)
PYREVIT_FILE_PREFIX_UNIVERSAL_USER_REGEX = \
    r'^' + PYREVIT_ADDON_NAME + r'_(?P<user>.+)_(?P<fname>.+)'

# e.g. pyRevit_2018_eirannejad_
PYREVIT_FILE_PREFIX_USER = '{}_{}_{}_'.format(PYREVIT_ADDON_NAME,
                                                HOST_APP.version,
                                                HOST_APP.username)
PYREVIT_FILE_PREFIX_USER_REGEX = \
    r'^' + PYREVIT_ADDON_NAME \
    + r'_(?P<version>\d{4})_(?P<user>.+)_(?P<fname>.+)'

# e.g. pyRevit_2018_eirannejad_14422_
PYREVIT_FILE_PREFIX_STAMPED_USER = '{}_{}_{}_{}_'.format(PYREVIT_ADDON_NAME,
                                                            HOST_APP.version,
                                                            HOST_APP.username,
                                                            HOST_APP.proc_id)
PYREVIT_FILE_PREFIX_STAMPED_USER_REGEX = \
    r'^' + PYREVIT_ADDON_NAME \
    + r'_(?P<version>\d{4})_(?P<user>.+)_(?P<pid>\d+)_(?P<fname>.+)'

# -----------------------------------------------------------------------------
# config labs modules
# -----------------------------------------------------------------------------
from pyrevit import labs



class ProcessThread(Component,IComponent,IDisposable):
 """ Represents an operating system process thread. """
 def Dispose(self):
  """
  Dispose(self: Component,disposing: bool)

   Releases the unmanaged resources used by the System.ComponentModel.Component and optionally 

    releases the managed resources.

  

  

   disposing: true to release both managed and unmanaged resources; false to release only unmanaged resources.
  """
  pass
 def GetService(self,*args):
  """
  GetService(self: Component,service: Type) -> object

  

   Returns an object that represents a service provided by the System.ComponentModel.Component or 

    by its System.ComponentModel.Container.

  

  

   service: A service provided by the System.ComponentModel.Component.

   Returns: An System.Object that represents a service provided by the System.ComponentModel.Component,or 

    null if the System.ComponentModel.Component does not provide the specified service.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def ResetIdealProcessor(self):
  """
  ResetIdealProcessor(self: ProcessThread)

   Resets the ideal processor for this thread to indicate that there is no single ideal processor. 

    In other words,so that any processor is ideal.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 BasePriority=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the base priority of the thread.



Get: BasePriority(self: ProcessThread) -> int



"""

 CanRaiseEvents=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the component can raise an event.



"""

 CurrentPriority=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current priority of the thread.



Get: CurrentPriority(self: ProcessThread) -> int



"""

 DesignMode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value that indicates whether the System.ComponentModel.Component is currently in design mode.



"""

 Events=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the list of event handlers that are attached to this System.ComponentModel.Component.



"""

 Id=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the unique identifier of the thread.



Get: Id(self: ProcessThread) -> int



"""

 IdealProcessor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Sets the preferred processor for this thread to run on.



Set: IdealProcessor(self: ProcessThread)=value

"""

 PriorityBoostEnabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the operating system should temporarily boost the priority of the associated thread whenever the main window of the thread's process receives the focus.



Get: PriorityBoostEnabled(self: ProcessThread) -> bool



Set: PriorityBoostEnabled(self: ProcessThread)=value

"""

 PriorityLevel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the priority level of the thread.



Get: PriorityLevel(self: ProcessThread) -> ThreadPriorityLevel



Set: PriorityLevel(self: ProcessThread)=value

"""

 PrivilegedProcessorTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the amount of time that the thread has spent running code inside the operating system core.



Get: PrivilegedProcessorTime(self: ProcessThread) -> TimeSpan



"""

 ProcessorAffinity=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Sets the processors on which the associated thread can run.



Set: ProcessorAffinity(self: ProcessThread)=value

"""

 StartAddress=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the memory address of the function that the operating system called that started this thread.



Get: StartAddress(self: ProcessThread) -> IntPtr



"""

 StartTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time that the operating system started the thread.



Get: StartTime(self: ProcessThread) -> DateTime



"""

 ThreadState=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the current state of this thread.



Get: ThreadState(self: ProcessThread) -> ThreadState



"""

 TotalProcessorTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the total amount of time that this thread has spent using the processor.



Get: TotalProcessorTime(self: ProcessThread) -> TimeSpan



"""

 UserProcessorTime=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the amount of time that the associated thread has spent running code inside the application.



Get: UserProcessorTime(self: ProcessThread) -> TimeSpan



"""

 WaitReason=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the reason that the thread is waiting.



Get: WaitReason(self: ProcessThread) -> ThreadWaitReason



"""





from typing import Tuple, Set, Iterable, List


class OccupancyUnit:
    NumberOfPeople = 0
    AreaPerPerson = 1
    BySpaceType = -1
    UseDefaultValues = -1


class DuctSystemType:
    UndefinedSystemType = 0
    SupplyAir = 1
    ReturnAir = 2
    ExhaustAir = 3
    OtherAir = 4
    Fitting = 28
    Global = 29


class SystemCalculationLevel:
    #None = 0
    Flow = 1
    Volume = 2
    Performance = 4
    All = -1


class RiseDropSymbol:
    NoSymbol = 0
    Outline = 1
    Cross = 2
    CrossFilled = 3
    Slash = 4
    SlashFilled = 5
    Backslash = 6
    BackslashFilled = 7
    Wye = 8
    ReverseWye = 9
    OutlineFilled = 10
    YinYang = 11
    YinYangFilled = 12
    BendThreeQuarterCircle = 13
    BendFullCircle = 14
    TeeHalfCircle = 15
    TeeFullCircle = 16
    CrossNoOutline = 17
    WyeFilled = 18
    ReverseWyeFilled = 19
    CustomSymbol = -1


class ReturnAirflowType:
    Specified = 0
    SpecifiedSupplyAirflow = 1
    CalculatedSupplyAirflow = 2
    ActualSupplyAirflow = 3


class ConditionType:
    Heated = 0
    Cooled = 1
    HeatedAndCooled = 2
    Unconditioned = 3
    Vented = 4
    NaturallyVentedOnly = 5
    NoOfConditionTypes = 6


class SpaceType:
    kActiveStorage = 0
    kActiveStorageHospitalOrHealthcare = 1
    kAirOrTrainOrBusBaggageArea = 2
    kAirportConcourse = 3
    kAtriumEachAdditionalFloor = 4
    kAtriumFirstThreeFloors = 5
    kAudienceOrSeatingAreaPenitentiary = 6
    kAudienceOrSeatingAreaExerciseCenter = 7
    kAudienceOrSeatingAreaGymnasium = 8
    kAudienceOrSeatingAreaSportsArena = 9
    kAudienceOrSeatingAreaConventionCenter = 10
    kAudienceOrSeatingAreaMotionPictureTheatre = 11
    kAudienceOrSeatingAreaPerformingArtsTheatre = 12
    kAudienceOrSeatingAreaReligious = 13
    kAudienceOrSeatingAreaPoliceOrFireStations = 14
    kAudienceOrSeatingAreaCourtHouse = 15
    kAudienceOrSeatingAreaAuditorium = 16
    kBankCustomerArea = 17
    kBankingActivityAreaOffice = 18
    kBarberAndBeautyParlor = 19
    kCardFileAndCataloguingLibrary = 20
    kClassroomOrLectureOrTrainingPenitentiary = 21
    kClassroomOrLectureOrTraining = 22
    kConfinementCellsPenitentiary = 23
    kConfinementCellsCourtHouse = 24
    kConferenceMeetingOrMultipurpose = 25
    kCorridorOrTransition = 26
    kCorridorOrTransitionManufacturingFacility = 27
    kCorridorsWithPatientWaitingExamHospitalOrHealthcare = 28
    kCourtSportsAreaSportsArena = 29
    kCourtroomCourtHouse = 30
    kDepartmentStoreSalesAreaRetail = 31
    kDetailedManufacturingFacility = 32
    kDiningArea = 33
    kDiningAreaHotel = 34
    kDiningAreaFamilyDining = 35
    kDiningAreaLoungeOrLeisureDining = 36
    kDiningAreaMotel = 37
    kDiningAreaTransportation = 38
    kDiningAreaPenitentiary = 39
    kDiningAreaCivilServices = 40
    kDormitoryBedroom = 41
    kDormitoryStudyHall = 42
    kDressingOrLockerOrFittingRoomGymnasium = 43
    kDressingOrLockerOrFittingRoomCourtHouse = 44
    kDressingOrLockerOrFittingRoomPerformingArtsTheatre = 45
    kDressingOrLockerOrFittingRoomAuditorium = 46
    kDressingOrLockerOrFittingRoomExerciseCenter = 47
    kElectricalOrMechanical = 48
    kElevatorLobbies = 49
    kEmergencyHospitalOrHealthcare = 50
    kEquipmentRoomManufacturingFacility = 51
    kExamOrTreatmentHospitalOrHealthcare = 52
    kExerciseAreaExerciseCenter = 53
    kExerciseAreaGymnasium = 54
    kExhibitSpaceConventionCenter = 55
    kFellowshipHallReligiousBuildings = 56
    kFineMaterialWarehouse = 57
    kFineMerchandiseSalesAreaRetail = 58
    kFireStationEngineRoomPoliceOrFireStation = 59
    kFoodPreparation = 60
    kGarageServiceOrRepairAutomotiveFacility = 61
    kGeneralHighBayManufacturingFacility = 62
    kGeneralLowBayManufacturingFacility = 63
    kGeneralExhibitionMuseum = 64
    kHospitalNurseryHospitalOrHealthcare = 65
    kHospitalOrMedicalSuppliesHospitalOrHealthcare = 66
    kHospitalOrRadiologyHospitalOrHealthcare = 67
    kHotelOrConferenceCenterConferenceOrMeeting = 68
    kInactiveStorage = 69
    kJudgesChambersCourtHouse = 70
    kLaboratoryOffice = 71
    kLaundryIroningAndSorting = 72
    kLaundryWashingHospitalOrHealthcare = 73
    kLibraryAudioVisualLibraryAudioVisual = 74
    kLivingQuartersDormitory = 75
    kLivingQuartersMotel = 76
    kLivingQuartersHotel = 77
    kLobby = 78
    kLobbyReligiousBuildings = 79
    kLobbyMotionPictureTheatre = 80
    kLobbyAuditorium = 81
    kLobbyPerformingArtsTheatre = 82
    kLobbyPostOffice = 83
    kLobbyHotel = 84
    kLoungeOrRecreation = 85
    kMallConcourseSalesAreaRetail = 86
    kMassMerchandisingSalesAreaRetail = 87
    kMediumOrBulkyMaterialWarehouse = 88
    kMerchandisingSalesAreaRetail = 89
    kMuseumAndGalleryStorage = 90
    kNurseStationHospitalOrHealthcare = 91
    kOfficeEnclosed = 92
    kOfficeOpenPlan = 93
    kOfficeCommonActivityAreasInactiveStorage = 94
    kOperatingRoomHospitalOrHealthcare = 95
    kOtherTelevisedPlayingAreaSportsArena = 96
    kParkingAreaAttendantOnlyParkingGarage = 97
    kParkingAreaPedestrianParkingGarage = 98
    kPatientRoomHospitalOrHealthcare = 99
    kPersonalServicesSalesAreaRetail = 100
    kPharmacyHospitalOrHealthcare = 101
    kPhysicalTherapyHospitalOrHealthcare = 102
    kPlayingAreaGymnasium = 103
    kPlenum = 104
    kPoliceStationLaboratoryPoliceOrFireStations = 105
    kPublicAndStaffLoungeHospitalOrHealthcare = 106
    kReadingAreaLibrary = 107
    kReceptionOrWaitingTransportation = 108
    kReceptionOrWaitingMotel = 109
    kReceptionOrWaitingHotel = 110
    kRecoveryHospitalOrHealthcare = 111
    kRestorationMuseum = 112
    kRestrooms = 113
    kRingSportsAreaSportsArena = 114
    kSleepingQuartersPoliceOrFireStation = 115
    kSortingAreaPostOffice = 116
    kSpecialtyStoreSalesAreaRetail = 117
    kStacksLibrary = 118
    kStairsInactive = 119
    kStairway = 120
    kSupermarketSalesAreaRetail = 121
    kTerminalTicketCounterTransportation = 122
    kWorkshopWorkshop = 123
    kWorshipPulpitChoirReligious = 124
    kNoOfSpaceTypes = 125
    NoSpaceType = -1


class DuctFlowConfigurationType:
    Calculated = 0
    Preset = 1
    System = 2


class DuctLossMethodType:
    NotDefined = 0
    SpecificLoss = 4
    Coefficient = 6


class SpaceSet(APIObject):
    def __init__(self): ...
    def Clear(self) -> None: ...
    @property
    def Size(self) -> int: ...
    @property
    def IsEmpty(self) -> bool: ...
    def ForwardIterator(self) -> SpaceSetIterator: ...
    def ReverseIterator(self) -> SpaceSetIterator: ...
    def GetEnumerator(self) -> IEnumerator: ...
    def Contains(self, item: Space) -> bool: ...
    def Insert(self, item: Space) -> bool: ...
    def Erase(self, item: Space) -> int: ...


class SpaceSetIterator(APIObject):
    def __init__(self): ...
    @property
    def Current(self) -> Object: ...
    def MoveNext(self) -> bool: ...
    def Reset(self) -> None: ...


class Space(SpatialElement):
    @property
    def Zone(self) -> Zone: ...
    @property
    def ClosedShell(self) -> GeometryElement: ...
    @property
    def Room(self) -> Room: ...
    @property
    def UpperLimit(self) -> Level: ...
    @UpperLimit.setter
    def UpperLimit(self, plev: Level) -> None: ...
    @property
    def LimitOffset(self) -> float: ...
    @LimitOffset.setter
    def LimitOffset(self, vLimitOffset: float) -> None: ...
    @property
    def BaseOffset(self) -> float: ...
    @BaseOffset.setter
    def BaseOffset(self, vBaseOffset: float) -> None: ...
    @property
    def AverageEstimatedIllumination(self) -> float: ...
    @property
    def SpaceCavityRatio(self) -> float: ...
    @property
    def LightingCalculationWorkplane(self) -> float: ...
    @LightingCalculationWorkplane.setter
    def LightingCalculationWorkplane(self, vLightingcw: float) -> None: ...
    @property
    def CeilingReflectance(self) -> float: ...
    @CeilingReflectance.setter
    def CeilingReflectance(self, vCeilingReflect: float) -> None: ...
    @property
    def WallReflectance(self) -> float: ...
    @WallReflectance.setter
    def WallReflectance(self, vWallReflect: float) -> None: ...
    @property
    def FloorReflectance(self) -> float: ...
    @FloorReflectance.setter
    def FloorReflectance(self, vFloorReflect: float) -> None: ...
    @property
    def DesignHVACLoadperArea(self) -> float: ...
    @DesignHVACLoadperArea.setter
    def DesignHVACLoadperArea(self, vDesignHVACLoadperArea: float) -> None: ...
    @property
    def ActualHVACLoad(self) -> float: ...
    @property
    def DesignOtherLoadperArea(self) -> float: ...
    @DesignOtherLoadperArea.setter
    def DesignOtherLoadperArea(self, vDesignOtherLoadperArea: float) -> None: ...
    @property
    def ActualOtherLoad(self) -> float: ...
    @property
    def DesignSupplyAirflow(self) -> float: ...
    @DesignSupplyAirflow.setter
    def DesignSupplyAirflow(self, vDesignSupplyAirflow: float) -> None: ...
    @property
    def CalculatedSupplyAirflow(self) -> float: ...
    @property
    def ActualSupplyAirflow(self) -> float: ...
    @property
    def ReturnAirflow(self) -> ReturnAirflowType: ...
    @ReturnAirflow.setter
    def ReturnAirflow(self, eReturnAirflow: ReturnAirflowType) -> None: ...
    @property
    def DesignReturnAirflow(self) -> float: ...
    @DesignReturnAirflow.setter
    def DesignReturnAirflow(self, vDesignReturnAirflow: float) -> None: ...
    @property
    def ActualReturnAirflow(self) -> float: ...
    @property
    def DesignExhaustAirflow(self) -> float: ...
    @DesignExhaustAirflow.setter
    def DesignExhaustAirflow(self, vDesignExhaustAirflow: float) -> None: ...
    @property
    def ActualExhaustAirflow(self) -> float: ...
    @property
    def OutdoorAirPerPerson(self) -> float: ...
    @property
    def OutdoorAirPerArea(self) -> float: ...
    @property
    def AirChangesPerHour(self) -> float: ...
    @property
    def OutdoorAirflow(self) -> float: ...
    @property
    def OutdoorAirFlowStandard(self) -> OutdoorAirFlowStandard: ...
    @property
    def UnboundedHeight(self) -> float: ...
    @property
    def Volume(self) -> float: ...
    @property
    def ConditionType(self) -> ConditionType: ...
    @ConditionType.setter
    def ConditionType(self, eConditionType: ConditionType) -> None: ...
    @property
    def SpaceType(self) -> SpaceType: ...
    @SpaceType.setter
    def SpaceType(self, eSpaceType: SpaceType) -> None: ...
    @property
    def SpaceTypeId(self) -> ElementId: ...
    @SpaceTypeId.setter
    def SpaceTypeId(self, spaceTypeId: ElementId) -> None: ...
    @property
    def SpaceConstruction(self) -> MEPSpaceConstruction: ...
    @property
    def CalculatedHeatingLoad(self) -> float: ...
    @property
    def DesignHeatingLoad(self) -> float: ...
    @DesignHeatingLoad.setter
    def DesignHeatingLoad(self, vDesignHeatingLoad: float) -> None: ...
    @property
    def CalculatedCoolingLoad(self) -> float: ...
    @property
    def DesignCoolingLoad(self) -> float: ...
    @DesignCoolingLoad.setter
    def DesignCoolingLoad(self, vDesignCoolingLoad: float) -> None: ...
    @property
    def OccupancyUnit(self) -> OccupancyUnit: ...
    @OccupancyUnit.setter
    def OccupancyUnit(self, eOccupancyUnit: OccupancyUnit) -> None: ...
    @property
    def BaseHeatLoadOn(self) -> BaseLoadOn: ...
    @BaseHeatLoadOn.setter
    def BaseHeatLoadOn(self, baseHeatLoadOn: BaseLoadOn) -> None: ...
    @property
    def NumberofPeople(self) -> float: ...
    @NumberofPeople.setter
    def NumberofPeople(self, vNumberofPeople: float) -> None: ...
    @property
    def AreaperPerson(self) -> float: ...
    @AreaperPerson.setter
    def AreaperPerson(self, vAreaperPerson: float) -> None: ...
    @property
    def SensibleHeatGainperPerson(self) -> float: ...
    @SensibleHeatGainperPerson.setter
    def SensibleHeatGainperPerson(self, vSensibleHeatGainperPerson: float) -> None: ...
    @property
    def LatentHeatGainperPerson(self) -> float: ...
    @LatentHeatGainperPerson.setter
    def LatentHeatGainperPerson(self, vLatentHeatGainperPerson: float) -> None: ...
    @property
    def LightingLoadUnit(self) -> BaseLoadOn: ...
    @LightingLoadUnit.setter
    def LightingLoadUnit(self, eLightingLoadUnit: BaseLoadOn) -> None: ...
    @property
    def ActualLightingLoad(self) -> float: ...
    @property
    def DesignLightingLoad(self) -> float: ...
    @DesignLightingLoad.setter
    def DesignLightingLoad(self, vDesignLightingLoad: float) -> None: ...
    @property
    def PowerLoadUnit(self) -> BaseLoadOn: ...
    @PowerLoadUnit.setter
    def PowerLoadUnit(self, ePowerLoadUnit: BaseLoadOn) -> None: ...
    @property
    def ActualPowerLoad(self) -> float: ...
    @property
    def DesignPowerLoad(self) -> float: ...
    @DesignPowerLoad.setter
    def DesignPowerLoad(self, vDesignPowerLoad: float) -> None: ...
    @property
    def Occupiable(self) -> bool: ...
    @property
    def Plenum(self) -> bool: ...
    def IsPointInSpace(self, point: XYZ) -> bool: ...


class SpaceTagType(FamilySymbol):


class FlexDuctType(MEPCurveType):


class Duct(MEPCurve):
    @property
    def DuctType(self) -> DuctType: ...
    @DuctType.setter
    def DuctType(self, ductType: DuctType) -> None: ...
    @property
    def IsPlaceholder(self) -> bool: ...
    @overload
    def Create(document: Document, ductTypeId: ElementId, levelId: ElementId, startConnector: Connector, endConnector: Connector) -> Duct: ...
    @overload
    def Create(document: Document, ductTypeId: ElementId, levelId: ElementId, startConnector: Connector, endPoint: XYZ) -> Duct: ...
    @overload
    def Create(document: Document, systemTypeId: ElementId, ductTypeId: ElementId, levelId: ElementId, startPoint: XYZ, endPoint: XYZ) -> Duct: ...
    def CreatePlaceholder(document: Document, systemTypeId: ElementId, ductTypeId: ElementId, levelId: ElementId, startPoint: XYZ, endPoint: XYZ) -> Duct: ...
    def SetSystemType(self, systemTypeId: ElementId) -> None: ...
    def IsHvacSystemTypeId(document: Document, systemTypeId: ElementId) -> bool: ...
    def IsDuctTypeId(document: Document, ductTypeId: ElementId) -> bool: ...


class DuctType(MEPCurveType):


class FlexDuct(MEPCurve):
    @property
    def Points(self) -> List[XYZ]: ...
    @Points.setter
    def Points(self, points: List[XYZ]) -> None: ...
    @property
    def FlexDuctType(self) -> FlexDuctType: ...
    @FlexDuctType.setter
    def FlexDuctType(self, flexDuctType: FlexDuctType) -> None: ...
    @property
    def StartTangent(self) -> XYZ: ...
    @StartTangent.setter
    def StartTangent(self, tangent: XYZ) -> None: ...
    @property
    def EndTangent(self) -> XYZ: ...
    @EndTangent.setter
    def EndTangent(self, tangent: XYZ) -> None: ...
    @overload
    def Create(document: Document, systemTypeId: ElementId, ductTypeId: ElementId, levelId: ElementId, startTangent: XYZ, endTangent: XYZ, points: List[XYZ]) -> FlexDuct: ...
    @overload
    def Create(document: Document, systemTypeId: ElementId, ductTypeId: ElementId, levelId: ElementId, points: List[XYZ]) -> FlexDuct: ...
    def IsFlexDuctTypeId(document: Document, ductTypeId: ElementId) -> bool: ...
    def IsHVACSystemTypeId(document: Document, systemTypeId: ElementId) -> bool: ...


class MechanicalSystem(MEPSystem):
    @property
    def SystemType(self) -> DuctSystemType: ...
    @property
    def BaseEquipmentConnector(self) -> Connector: ...
    @BaseEquipmentConnector.setter
    def BaseEquipmentConnector(self, baseEquipmentConnector: Connector) -> None: ...
    @property
    def DuctNetwork(self) -> ElementSet: ...
    @property
    def IsWellConnected(self) -> bool: ...
    def IsPressureDropServerMissing(self) -> bool: ...
    @overload
    def Create(ADocument: Document, typeId: ElementId, name: str) -> MechanicalSystem: ...
    @overload
    def Create(ADocument: Document, typeId: ElementId) -> MechanicalSystem: ...
    def GetFlow(self) -> float: ...
    def GetStaticPressure(self) -> float: ...


class SpaceTag(SpatialElementTag):
    @property
    def Space(self) -> Space: ...
    @property
    def SpaceTagType(self) -> SpaceTagType: ...
    @SpaceTagType.setter
    def SpaceTagType(self, A_0: SpaceTagType) -> None: ...


class Zone(Element):
    @Name.setter
    def Name(self, valueString: str) -> None: ...
    @property
    def Phase(self) -> Phase: ...
    @property
    def IsDefaultZone(self) -> bool: ...
    @IsDefaultZone.setter
    def IsDefaultZone(self, bDefaultZone: bool) -> None: ...
    @property
    def Area(self) -> float: ...
    @property
    def GrossArea(self) -> float: ...
    @property
    def Volume(self) -> float: ...
    @property
    def GrossVolume(self) -> float: ...
    @property
    def Perimeter(self) -> float: ...
    @property
    def Spaces(self) -> SpaceSet: ...
    @property
    def Boundary(self) -> CurveArray: ...
    @property
    def ServiceType(self) -> ServiceType: ...
    @ServiceType.setter
    def ServiceType(self, eSpaceService: ServiceType) -> None: ...
    @property
    def CalculatedSupplyAirflow(self) -> float: ...
    @property
    def CalculatedHeatingLoad(self) -> float: ...
    @property
    def CalculatedCoolingLoad(self) -> float: ...
    @property
    def HeatingSetPoint(self) -> float: ...
    @HeatingSetPoint.setter
    def HeatingSetPoint(self, vHeatingSetPoint: float) -> None: ...
    @property
    def CoolingSetPoint(self) -> float: ...
    @CoolingSetPoint.setter
    def CoolingSetPoint(self, vCoolingSetPoint: float) -> None: ...
    @property
    def HeatingAirTemperature(self) -> float: ...
    @HeatingAirTemperature.setter
    def HeatingAirTemperature(self, vHeatingAirTemperature: float) -> None: ...
    @property
    def CoolingAirTemperature(self) -> float: ...
    @CoolingAirTemperature.setter
    def CoolingAirTemperature(self, vCoolingAirTemperature: float) -> None: ...
    @property
    def HumidificationSetPoint(self) -> float: ...
    @HumidificationSetPoint.setter
    def HumidificationSetPoint(self, vHumidificationSetPoint: float) -> None: ...
    @property
    def DehumidificationSetPoint(self) -> float: ...
    @DehumidificationSetPoint.setter
    def DehumidificationSetPoint(self, vDehumidificationSetPoint: float) -> None: ...
    def AddSpaces(self, spaces: SpaceSet) -> bool: ...
    def RemoveSpaces(self, spaces: SpaceSet) -> bool: ...


class DuctSizeIterator:
    @property
    def IsValidObject(self) -> bool: ...
    def MoveNext(self) -> bool: ...
    def IsDone(self) -> bool: ...
    def Reset(self) -> None: ...
    def GetCurrent(self) -> MEPSize: ...
    def HasCurrent(self) -> bool: ...
    @property
    def Current(self) -> MEPSize: ...
    def Dispose(self) -> None: ...


class DuctSizes:
    def GetDuctSizeIterator(self) -> DuctSizeIterator: ...
    @property
    def Count(self) -> int: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Contains(self, nominalDiameter: float) -> bool: ...
    def GetEnumerator(self) -> IEnumerator: ...
    def Dispose(self) -> None: ...


class DuctSizeSettingIterator:
    @property
    def IsValidObject(self) -> bool: ...
    def MoveNext(self) -> bool: ...
    def IsDone(self) -> bool: ...
    def Reset(self) -> None: ...
    def HasCurrent(self) -> bool: ...
    @property
    def Current(self) -> KeyValuePair: ...
    def Dispose(self) -> None: ...


class DuctSizeSettings(Element):
    def GetDuctSizeSettingIterator(self) -> DuctSizeSettingIterator: ...
    @property
    def Item(self, ductShape: DuctShape) -> DuctSizes: ...
    def GetSizeCount(self, shape: DuctShape) -> int: ...
    def GetDuctSizeSettings(aDoc: Document) -> DuctSizeSettings: ...
    def AddSize(self, shape: DuctShape, sizeInfo: MEPSize) -> None: ...
    def RemoveSize(self, shape: DuctShape, nominalDiameter: float) -> None: ...
    def GetEnumerator(self) -> IEnumerator: ...


class DuctShape:
    Round = 0
    Rectangular = 1
    Oval = 2


class MechanicalEquipment(MEPModel):


class MechanicalFitting(MEPModel):
    @property
    def PartType(self) -> PartType: ...


class ComponentClassification:
    Undefined = 0
    Pipe = 1
    Duct = 2
    FlexPipe = 11
    FlexDuct = 12
    Elbow = 101
    Tee = 102
    Tap = 103
    Transition = 104
    Cross = 105
    Endcap = 106
    Coupling = 107
    Union = 108
    Flange = 109
    Wye = 110
    Valve = 111
    Sensor = 112
    Hanger = 113
    Sleeve = 114


class MEPBuildingConstruction(ElementType):
    def GetConstructions(self, constructionType: ConstructionType) -> ICollection: ...
    def GetBuildingConstruction(self, constructionType: ConstructionType) -> Construction: ...
    def SetBuildingConstruction(self, constructionType: ConstructionType, buildingConstruction: Construction) -> None: ...
    def SetBuildingConstructionOverride(self, constructionType: ConstructionType, override: bool) -> None: ...
    def GetBuildingConstructionOverride(self, constructionType: ConstructionType) -> bool: ...


class MEPBuildingConstructionSet(APIObject):
    def __init__(self): ...
    def Clear(self) -> None: ...
    @property
    def Size(self) -> int: ...
    @property
    def IsEmpty(self) -> bool: ...
    def ForwardIterator(self) -> MEPBuildingConstructionSetIterator: ...
    def ReverseIterator(self) -> MEPBuildingConstructionSetIterator: ...
    def GetEnumerator(self) -> IEnumerator: ...
    def Contains(self, item: MEPBuildingConstruction) -> bool: ...
    def Insert(self, item: MEPBuildingConstruction) -> bool: ...
    def Erase(self, item: MEPBuildingConstruction) -> int: ...


class MEPBuildingConstructionSetIterator(APIObject):
    def __init__(self): ...
    @property
    def Current(self) -> Object: ...
    def MoveNext(self) -> bool: ...
    def Reset(self) -> None: ...


class MEPSpaceConstruction:
    @property
    def CurrentConstruction(self) -> MEPBuildingConstruction: ...
    @CurrentConstruction.setter
    def CurrentConstruction(self, pCurrentConstruction: MEPBuildingConstruction) -> None: ...
    @property
    def SpaceConstructions(self) -> MEPBuildingConstructionSet: ...
    def NewConstruction(self, pName: str) -> MEPBuildingConstruction: ...
    def DuplicateConstruction(self, pCurrentConstruction: MEPBuildingConstruction, pName: str) -> MEPBuildingConstruction: ...
    def DeleteConstruction(self, pCurrentConstruction: MEPBuildingConstruction) -> None: ...


class AirSystemData:
    @property
    def HeatExchangerType(self) -> AirHeatExchangerType: ...
    @HeatExchangerType.setter
    def HeatExchangerType(self, type: AirHeatExchangerType) -> None: ...
    @property
    def PreheatCoilType(self) -> AirHeatingCoilType: ...
    @PreheatCoilType.setter
    def PreheatCoilType(self, type: AirHeatingCoilType) -> None: ...
    @property
    def PreheatHotWaterLoopId(self) -> ElementId: ...
    @PreheatHotWaterLoopId.setter
    def PreheatHotWaterLoopId(self, waterLoopId: ElementId) -> None: ...
    @property
    def HeatingCoilType(self) -> AirHeatingCoilType: ...
    @HeatingCoilType.setter
    def HeatingCoilType(self, type: AirHeatingCoilType) -> None: ...
    @property
    def HeatingHotWaterLoopId(self) -> ElementId: ...
    @HeatingHotWaterLoopId.setter
    def HeatingHotWaterLoopId(self, waterLoopId: ElementId) -> None: ...
    @property
    def CoolingCoilType(self) -> AirCoolingCoilType: ...
    @CoolingCoilType.setter
    def CoolingCoilType(self, type: AirCoolingCoilType) -> None: ...
    @property
    def ChilledWaterLoopId(self) -> ElementId: ...
    @ChilledWaterLoopId.setter
    def ChilledWaterLoopId(self, waterLoopId: ElementId) -> None: ...
    @property
    def AirFanType(self) -> AirFanType: ...
    @AirFanType.setter
    def AirFanType(self, type: AirFanType) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class EquipmentClassification:
    #None = 0
    Pump = 5


class AnalyticalSystemDomain:
    WaterLoop = 0
    AirSystem = 1


class WaterLoopType:
    Undefined = 0
    ChilledWater = 1
    CondenserWater = 2
    HotWater = 3
    VRF = 4


class WaterChillerType:
    AirCooled = 0
    WaterCooled = 1


class AirHeatExchangerType:
    #None = 0
    Enthalpy = 1
    Sensible = 2


class AirHeatingCoilType:
    #None = 0
    ElectricResistance = 1
    Furnace = 2
    HotWater = 3


class AirCoolingCoilType:
    #None = 0
    ChilledWater = 1
    DirectExpansion = 2


class AirFanType:
    ConstantVolume = 0
    VariableVolume = 1


class ZoneEquipmentHvacType:
    Undefined = 0
    ChilledBeamActive = 1
    FourPipeFanCoil = 2
    PackagedTerminalAirConditioner = 3
    PackagedTerminalHeatPump = 4
    ParallelFanPoweredBox = 5
    VAVBox = 6
    CAVBox = 9
    RadiantPanel = 10
    SeriesFanPoweredBox = 11
    UnitHeater = 12
    UnitVentilator = 13
    WaterSourceHeatPump = 14
    ChilledBeamPassive = 15
    Unconditioned = 16
    VRFFanCoil = 17


class ZoneEquipmentBehavior:
    OnePerSpace = 0
    GroupSpaces = 1


class DuctFittingAndAccessoryConnectorData:
    @property
    def Width(self) -> float: ...
    @property
    def Height(self) -> float: ...
    @property
    def Diameter(self) -> float: ...
    @property
    def Angle(self) -> float: ...
    @property
    def Index(self) -> int: ...
    @property
    def LinkIndex(self) -> int: ...
    @property
    def FlowDirection(self) -> FlowDirectionType: ...
    @property
    def Flow(self) -> float: ...
    @property
    def VelocityPressure(self) -> float: ...
    @property
    def Profile(self) -> ConnectorProfileType: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetCoordination(self) -> Transform: ...
    def Dispose(self) -> None: ...


class DuctFittingAndAccessoryData:
    @property
    def ServerGUID(self) -> Guid: ...
    @property
    def PartType(self) -> PartType: ...
    @property
    def SystemClassification(self) -> MEPSystemClassification: ...
    @property
    def Origin(self) -> XYZ: ...
    @property
    def AirViscosity(self) -> float: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetEntity(self) -> Entity: ...
    def GetAllConnectorData(self) -> List[DuctFittingAndAccessoryConnectorData]: ...
    def GetFamilyInstanceId(self) -> ElementId: ...
    def Dispose(self) -> None: ...


class DuctFittingAndAccessoryPressureDropItem:
    @property
    def BeginConnectorIndex(self) -> int: ...
    @property
    def EndConnectorIndex(self) -> int: ...
    @property
    def VelocityPressure(self) -> float: ...
    @property
    def Coefficient(self) -> float: ...
    @Coefficient.setter
    def Coefficient(self, coefficient: float) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class DuctFittingAndAccessoryPressureDropData:
    @property
    def CalculationType(self) -> int: ...
    @property
    def IsCurrentEntityValid(self) -> bool: ...
    @IsCurrentEntityValid.setter
    def IsCurrentEntityValid(self, isCurrentEntityValid: bool) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetDuctFittingAndAccessoryData(self) -> DuctFittingAndAccessoryData: ...
    def GetPresureDropItems(self) -> List[DuctFittingAndAccessoryPressureDropItem]: ...
    def SetDefaultEntity(self, defaultEntity: Entity) -> None: ...
    def Dispose(self) -> None: ...


class DuctPressureDropData:
    @property
    def Level(self) -> SystemCalculationLevel: ...
    @property
    def Shape(self) -> ConnectorProfileType: ...
    @property
    def CategoryId(self) -> ElementId: ...
    @property
    def Height(self) -> float: ...
    @property
    def WidthOrDiameter(self) -> float: ...
    @property
    def Length(self) -> float: ...
    @property
    def Density(self) -> float: ...
    @property
    def Viscosity(self) -> float: ...
    @property
    def Roughness(self) -> float: ...
    @property
    def Flow(self) -> float: ...
    @property
    def EquivalentDiameter(self) -> float: ...
    @EquivalentDiameter.setter
    def EquivalentDiameter(self, equivalentDiameter: float) -> None: ...
    @property
    def HydraulicDiameter(self) -> float: ...
    @HydraulicDiameter.setter
    def HydraulicDiameter(self, hydraulicDiameter: float) -> None: ...
    @property
    def ReynoldsNumber(self) -> float: ...
    @ReynoldsNumber.setter
    def ReynoldsNumber(self, reynoldsNumber: float) -> None: ...
    @property
    def Velocity(self) -> float: ...
    @Velocity.setter
    def Velocity(self, velocity: float) -> None: ...
    @property
    def VelocityPressure(self) -> float: ...
    @VelocityPressure.setter
    def VelocityPressure(self, velocityPressure: float) -> None: ...
    @property
    def Friction(self) -> float: ...
    @Friction.setter
    def Friction(self, friction: float) -> None: ...
    @property
    def PressureDrop(self) -> float: ...
    @PressureDrop.setter
    def PressureDrop(self, pressureDrop: float) -> None: ...
    @property
    def Coefficient(self) -> float: ...
    @Coefficient.setter
    def Coefficient(self, coefficient: float) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class IDuctFittingAndAccessoryPressureDropServer:
    def Calculate(self, data: DuctFittingAndAccessoryPressureDropData) -> bool: ...
    def IsApplicable(self, data: DuctFittingAndAccessoryPressureDropData) -> bool: ...
    def GetDataSchema(self) -> Schema: ...


class IDuctPressureDropServer:
    def Calculate(self, data: DuctPressureDropData) -> None: ...
    def GetInformationLink(self) -> str: ...
    def GetHtmlDescription(self) -> str: ...


class MEPAnalyticalSystem(Element):
    @property
    def AnalyticalSystemDomain(self) -> AnalyticalSystemDomain: ...
    def Create(document: Document, domain: AnalyticalSystemDomain, name: str) -> MEPAnalyticalSystem: ...
    def GetWaterLoopData(self) -> WaterLoopData: ...
    def GetAirSystemData(self) -> AirSystemData: ...


class WaterLoopData:
    @property
    def WaterLoopType(self) -> WaterLoopType: ...
    @WaterLoopType.setter
    def WaterLoopType(self, type: WaterLoopType) -> None: ...
    @property
    def ChillerType(self) -> WaterChillerType: ...
    @ChillerType.setter
    def ChillerType(self, type: WaterChillerType) -> None: ...
    @property
    def CondenserWaterLoopId(self) -> ElementId: ...
    @CondenserWaterLoopId.setter
    def CondenserWaterLoopId(self, waterLoopElementId: ElementId) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def Dispose(self) -> None: ...


class SystemZoneData(GenericZoneDomainData):
    @property
    def ZoneEquipmentId(self) -> ElementId: ...
    @ZoneEquipmentId.setter
    def ZoneEquipmentId(self, id: ElementId) -> None: ...
    def Create() -> SystemZoneData: ...


class MechanicalEquipmentSet(Element):
    @property
    def Classification(self) -> EquipmentClassification: ...
    @property
    def OnDuty(self) -> int: ...
    @OnDuty.setter
    def OnDuty(self, number: int) -> None: ...
    @property
    def OnStandby(self) -> int: ...
    @OnStandby.setter
    def OnStandby(self, number: int) -> None: ...
    def Create(document: Document, typeId: ElementId, memberIds: ISet) -> MechanicalEquipmentSet: ...
    def AreElementsNotConnectedInSeries(document: Document, elemIds: ISet) -> bool: ...
    def AreValidMembers(document: Document, memberIds: ISet) -> bool: ...
    def GetMembers(self) -> ISet: ...
    def Add(self, elemIds: ISet) -> None: ...
    def Remove(self, elemIds: ISet) -> None: ...


class MechanicalEquipmentSetType(ElementType):
    def Create(document: Document, name: str) -> MechanicalEquipmentSetType: ...


class MechanicalUtils:
    def ConvertDuctPlaceholders(document: Document, placeholderIds: ICollection) -> ICollection: ...
    @overload
    def ConnectDuctPlaceholdersAtElbow(document: Document, connector1: Connector, connector2: Connector) -> bool: ...
    @overload
    def ConnectDuctPlaceholdersAtElbow(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId) -> bool: ...
    @overload
    def ConnectDuctPlaceholdersAtTee(document: Document, connector1: Connector, connector2: Connector, connector3: Connector) -> bool: ...
    @overload
    def ConnectDuctPlaceholdersAtTee(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId) -> bool: ...
    @overload
    def ConnectDuctPlaceholdersAtCross(document: Document, connector1: Connector, connector2: Connector, connector3: Connector, connector4: Connector) -> bool: ...
    @overload
    def ConnectDuctPlaceholdersAtCross(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId, placeholder3Id: ElementId) -> bool: ...
    @overload
    def ConnectDuctPlaceholdersAtCross(document: Document, placeholder1Id: ElementId, placeholder2Id: ElementId) -> bool: ...
    def ConnectAirTerminalOnDuct(document: Document, airTerminalId: ElementId, ductCurveId: ElementId) -> bool: ...
    def BreakCurve(document: Document, ductId: ElementId, ptBreak: XYZ) -> ElementId: ...


class DuctInsulation(InsulationLiningBase):
    def Create(document: Document, ductOrContentElementId: ElementId, ductInsulationTypeId: ElementId, Thickness: float) -> DuctInsulation: ...


class DuctInsulationType(ElementType):


class DuctLining(InsulationLiningBase):
    def Create(document: Document, ductOrContentElementId: ElementId, ductLiningTypeId: ElementId, Thickness: float) -> DuctLining: ...


class DuctLiningType(ElementType):
    @property
    def Roughness(self) -> float: ...
    @Roughness.setter
    def Roughness(self, roughness: float) -> None: ...
    def IsValidRoughness(self, roughness: float) -> bool: ...


class DuctSettings(Element):
    @property
    def RectangularDuctSizeSeparator(self) -> str: ...
    @RectangularDuctSizeSeparator.setter
    def RectangularDuctSizeSeparator(self, rectangularDuctSizeSeparator: str) -> None: ...
    @property
    def RectangularDuctSizeSuffix(self) -> str: ...
    @RectangularDuctSizeSuffix.setter
    def RectangularDuctSizeSuffix(self, rectangularDuctSizeSuffix: str) -> None: ...
    @property
    def RoundDuctSizeSuffix(self) -> str: ...
    @RoundDuctSizeSuffix.setter
    def RoundDuctSizeSuffix(self, roundDuctSizeSuffix: str) -> None: ...
    @property
    def RoundDuctSizePrefix(self) -> str: ...
    @RoundDuctSizePrefix.setter
    def RoundDuctSizePrefix(self, roundDuctSizePrefix: str) -> None: ...
    @property
    def ConnectorSeparator(self) -> str: ...
    @ConnectorSeparator.setter
    def ConnectorSeparator(self, connectorSeparator: str) -> None: ...
    @property
    def OvalDuctSizeSeparator(self) -> str: ...
    @OvalDuctSizeSeparator.setter
    def OvalDuctSizeSeparator(self, ovalDuctSizeSeparator: str) -> None: ...
    @property
    def OvalDuctSizeSuffix(self) -> str: ...
    @OvalDuctSizeSuffix.setter
    def OvalDuctSizeSuffix(self, ovalDuctSizeSuffix: str) -> None: ...
    @property
    def FlatOnTop(self) -> str: ...
    @FlatOnTop.setter
    def FlatOnTop(self, flatOnTop: str) -> None: ...
    @property
    def FlatOnBottom(self) -> str: ...
    @FlatOnBottom.setter
    def FlatOnBottom(self, flatOnBottom: str) -> None: ...
    @property
    def SetUp(self) -> str: ...
    @SetUp.setter
    def SetUp(self, setUp: str) -> None: ...
    @property
    def SetDown(self) -> str: ...
    @SetDown.setter
    def SetDown(self, setDown: str) -> None: ...
    @property
    def SetUpFromBottom(self) -> str: ...
    @SetUpFromBottom.setter
    def SetUpFromBottom(self, setUpFromBottom: str) -> None: ...
    @property
    def SetDownFromBottom(self) -> str: ...
    @SetDownFromBottom.setter
    def SetDownFromBottom(self, setDownFromBottom: str) -> None: ...
    @property
    def Centerline(self) -> str: ...
    @Centerline.setter
    def Centerline(self, centerline: str) -> None: ...
    @property
    def AirDensity(self) -> float: ...
    @AirDensity.setter
    def AirDensity(self, airDensity: float) -> None: ...
    @property
    def AirViscosity(self) -> float: ...
    @AirViscosity.setter
    def AirViscosity(self, airViscosity: float) -> None: ...
    @property
    def FittingAnnotationSize(self) -> float: ...
    @FittingAnnotationSize.setter
    def FittingAnnotationSize(self, fittingAnnotationSize: float) -> None: ...
    @property
    def RiseDropAnnotationSize(self) -> float: ...
    @RiseDropAnnotationSize.setter
    def RiseDropAnnotationSize(self, riseDropAnnotationSize: float) -> None: ...
    @property
    def FittingAngleUsage(self) -> FittingAngleUsage: ...
    @FittingAngleUsage.setter
    def FittingAngleUsage(self, fittingAngleUsage: FittingAngleUsage) -> None: ...
    @property
    def UseAnnotationScaleForSingleLineFittings(self) -> bool: ...
    @UseAnnotationScaleForSingleLineFittings.setter
    def UseAnnotationScaleForSingleLineFittings(self, useAnnotationScaleForSingleLineFittings: bool) -> None: ...
    def GetDuctSettings(document: Document) -> DuctSettings: ...
    def GetSpecificFittingAngles(self) -> List[float]: ...
    def SetSpecificFittingAngleStatus(self, angle: float, useInLayout: bool) -> None: ...
    def GetSpecificFittingAngleStatus(self, angle: float) -> bool: ...
    def IsValidSpecificFittingAngle(self, angle: float) -> bool: ...
    def GetPressLossCalculationServerInfo(self) -> MEPCalculationServerInfo: ...
    def SetPressLossCalculationServerInfo(self, serverInfo: MEPCalculationServerInfo) -> None: ...


class MechanicalSystemType(MEPSystemType):
    @property
    def RiseDropSettings(self) -> RiseDropSymbol: ...
    @RiseDropSettings.setter
    def RiseDropSettings(self, RiseDropSettings: RiseDropSymbol) -> None: ...
    def Create(ADoc: Document, systemClassification: MEPSystemClassification, name: str) -> MechanicalSystemType: ...
    def ValidateRiseDropSymbolType(self, risedropType: RiseDropSymbol) -> bool: ...


class MEPSection:
    @property
    def Number(self) -> int: ...
    @property
    def Roughness(self) -> float: ...
    @property
    def Flow(self) -> float: ...
    @property
    def FixtureUnit(self) -> float: ...
    @property
    def Friction(self) -> float: ...
    @property
    def Velocity(self) -> float: ...
    @property
    def VelocityPressure(self) -> float: ...
    @property
    def TotalCoefficient(self) -> float: ...
    @property
    def TotalPressureLoss(self) -> float: ...
    @property
    def TotalCurveLength(self) -> float: ...
    @property
    def ReynoldsNumber(self) -> float: ...
    @property
    def FrictionFactor(self) -> float: ...
    @property
    def IsValidObject(self) -> bool: ...
    def GetPressureDrop(self, elemId: ElementId) -> float: ...
    def GetCoefficient(self, elemId: ElementId) -> float: ...
    def GetSegmentLength(self, segmentId: ElementId) -> float: ...
    def IsMain(self, fittingId: ElementId) -> bool: ...
    def GetElementIds(self) -> List[ElementId]: ...
    def Dispose(self) -> None: ...


class SpaceFilter(ElementSlowFilter):
    def __init__(self): ...


class SpaceTagFilter(ElementSlowFilter):
    def __init__(self): ...


class ZoneEquipment(Element):
    def Create(document: Document, name: str) -> ZoneEquipment: ...
    def GetZoneEquipmentData(self) -> ZoneEquipmentData: ...
    def MoveSpaceToEquipment(document: Document, analyticalSpaceSet: ISet, originalZoneEquipmentId: ElementId, targetZoneEquipmentId: ElementId) -> None: ...
    @overload
    def GetAssociatedZoneEquipment(document: Document, spaces: ISet) -> ISet: ...
    @overload
    def GetAssociatedZoneEquipment(document: Document, spaceElementId: ElementId) -> ISet: ...


class ZoneEquipmentData:
    @property
    def EquipmentType(self) -> ZoneEquipmentHvacType: ...
    @EquipmentType.setter
    def EquipmentType(self, type: ZoneEquipmentHvacType) -> None: ...
    @property
    def EquipmentBehavior(self) -> ZoneEquipmentBehavior: ...
    @EquipmentBehavior.setter
    def EquipmentBehavior(self, type: ZoneEquipmentBehavior) -> None: ...
    @property
    def HeatingCoilType(self) -> AirHeatingCoilType: ...
    @HeatingCoilType.setter
    def HeatingCoilType(self, type: AirHeatingCoilType) -> None: ...
    @property
    def CoolingCoilType(self) -> AirCoolingCoilType: ...
    @CoolingCoilType.setter
    def CoolingCoilType(self, type: AirCoolingCoilType) -> None: ...
    @property
    def HotWaterLoopId(self) -> ElementId: ...
    @HotWaterLoopId.setter
    def HotWaterLoopId(self, waterLoopId: ElementId) -> None: ...
    @property
    def CondenserWaterLoopId(self) -> ElementId: ...
    @CondenserWaterLoopId.setter
    def CondenserWaterLoopId(self, waterLoopId: ElementId) -> None: ...
    @property
    def ChilledWaterLoopId(self) -> ElementId: ...
    @ChilledWaterLoopId.setter
    def ChilledWaterLoopId(self, waterLoopId: ElementId) -> None: ...
    @property
    def VRFLoopId(self) -> ElementId: ...
    @VRFLoopId.setter
    def VRFLoopId(self, vrfLoopId: ElementId) -> None: ...
    @property
    def AirSystemId(self) -> ElementId: ...
    @AirSystemId.setter
    def AirSystemId(self, airLoopId: ElementId) -> None: ...
    @property
    def IsValidObject(self) -> bool: ...
    def IsDataCompleted(self) -> bool: ...
    def Dispose(self) -> None: ...



class ComUnregisterFunctionAttribute(Attribute,_Attribute):
 """
 Specifies the method to call when you unregister an assembly for use from COM; this allows for the execution of user-written code during the unregistration process.

 

 ComUnregisterFunctionAttribute()
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class WebBrowserDocumentCompletedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Forms.WebBrowser.DocumentCompleted event.

 

 WebBrowserDocumentCompletedEventArgs(url: Uri)
 """
 @staticmethod
 def __new__(self,url):
  """ __new__(cls: type,url: Uri) """
  pass
 Url=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location of the document to which the System.Windows.Forms.WebBrowser control has navigated.



Get: Url(self: WebBrowserDocumentCompletedEventArgs) -> Uri



"""





class ResourceType(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the defined native object types.

 

 enum ResourceType,values: DSObject (8),DSObjectAll (9),FileObject (1),KernelObject (6),LMShare (5),Printer (3),ProviderDefined (10),RegistryKey (4),RegistryWow6432Key (12),Service (2),Unknown (0),WindowObject (7),WmiGuidObject (11)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 DSObject=None
 DSObjectAll=None
 FileObject=None
 KernelObject=None
 LMShare=None
 Printer=None
 ProviderDefined=None
 RegistryKey=None
 RegistryWow6432Key=None
 Service=None
 Unknown=None
 value__=None
 WindowObject=None
 WmiGuidObject=None




class ProtocolViolationException(InvalidOperationException,ISerializable,_Exception):
 """
 The exception that is thrown when an error is made while using a network protocol.

 

 ProtocolViolationException()

 ProtocolViolationException(message: str)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,serializationInfo,streamingContext):
  """
  GetObjectData(self: ProtocolViolationException,serializationInfo: SerializationInfo,streamingContext: StreamingContext)

   Populates a System.Runtime.Serialization.SerializationInfo with the data required to serialize 

    the target object.

  

  

   serializationInfo: The System.Runtime.Serialization.SerializationInfo to populate with data.

   streamingContext: A System.Runtime.Serialization.StreamingContext that specifies the destination for this 

    serialization.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None):
  """
  __new__(cls: type)

  __new__(cls: type,message: str)

  __new__(cls: type,serializationInfo: SerializationInfo,streamingContext: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass



class ThreadState(Enum,IComparable,IFormattable,IConvertible):
 """
 Specifies the current execution state of the thread.

 

 enum ThreadState,values: Initialized (0),Ready (1),Running (2),Standby (3),Terminated (4),Transition (6),Unknown (7),Wait (5)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 Initialized=None
 Ready=None
 Running=None
 Standby=None
 Terminated=None
 Transition=None
 Unknown=None
 value__=None
 Wait=None




"""Utilities to load and manage assemblies."""
import os.path as op

from pyrevit import PyRevitException
from pyrevit import framework
from pyrevit.compat import safe_strtype
from pyrevit.coreutils import logger


mlogger = logger.get_logger(__name__)


def load_asm(asm_name):
    """Load assembly by name into current domain.

    Args:
        asm_name (str): assembly name

    Returns:
        (Any): the loaded assembly, None if not loaded.
    """
    return framework.AppDomain.CurrentDomain.Load(asm_name)


def load_asm_file(asm_file):
    """Load assembly by file into current domain.

    Args:
        asm_file (str): assembly file path

    Returns:
        (Any): loaded assembly, None if not loaded.
    """
    try:
        return framework.Assembly.LoadFrom(asm_file)
    except Exception as load_ex:
        mlogger.error("Error loading assembly @ %s | %s", asm_file, load_ex)
        return None


def load_asm_files(asm_files):
    """Load assemblies by file into current domain.

    Args:
        asm_files (list[str]): list of assembly file paths
    """
    for asm_file in asm_files:
        load_asm_file(asm_file)


def find_loaded_asm(asm_info, by_partial_name=False, by_location=False):
    """Find loaded assembly based on name, partial name, or location.

    Args:
        asm_info (str): name or location of the assembly
        by_partial_name (bool): returns all assemblies that has the asm_info
        by_location (bool): returns all assemblies matching location

    Returns:
        (list): List of all loaded assemblies matching the provided info
            If only one assembly has been found, it returns the assembly.
            None will be returned if assembly is not loaded.
    """
    loaded_asm_list = []
    cleaned_asm_info = \
        asm_info.lower().replace('.' + framework.ASSEMBLY_FILE_TYPE, '')
    for loaded_assembly in framework.AppDomain.CurrentDomain.GetAssemblies():
        if by_partial_name:
            if cleaned_asm_info in \
                    safe_strtype(loaded_assembly.GetName().Name).lower():
                loaded_asm_list.append(loaded_assembly)
        elif by_location:
            try:
                if op.normpath(loaded_assembly.Location) == \
                        op.normpath(asm_info):
                    loaded_asm_list.append(loaded_assembly)
            except Exception:
                continue
        elif cleaned_asm_info == \
                safe_strtype(loaded_assembly.GetName().Name).lower():
            loaded_asm_list.append(loaded_assembly)

    return loaded_asm_list


def find_type_by_name(assembly, type_name):
    """Find type by name in assembly.

    Args:
        assembly (Assembly): assembly to find the type in
        type_name (str): type name

    Returns:
        (type): type if found.

    Raises:
        PyRevitException: if type not found.
    """
    base_class = assembly.GetType(type_name)
    if base_class is not None:
        return base_class
    else:
        raise PyRevitException('Can not find base class type: {}'
                               .format(type_name))
